#include "stdafx.h"
#include "BGBitmap.h"

////////////////////////////////////////////////////////////////////////////////////////////////
//Bitmap creation functions

bool CBGBitmap::LoadFromResource(UINT nIDResource)
{
    DeleteObject();
    if (nIDResource)
    {
        if ( CBitmap::LoadBitmap(nIDResource) )
        {
            BITMAP bm;
	        GetBitmap(&bm);
            SetBitmapDimension(bm.bmWidth,bm.bmHeight);
            return true;
        }
    }
    return false;
}

bool CBGBitmap::LoadFromFile(CString &strFile)
{
    DeleteObject();
    if (!strFile.IsEmpty())
    {
        HBITMAP hBitmap = (HBITMAP)::LoadImage(0, strFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
        if (hBitmap)
        {
            BITMAP bm;
            Attach(hBitmap);
	        GetBitmap(&bm);
            //Check size of bitmap, bitmap size must not exceed max allowed size!
            if (bm.bmWidth*bm.bmHeight>120000)
            {
                DeleteObject();
            } 
            else
            {
                SetBitmapDimension(bm.bmWidth,bm.bmHeight);
                return true;
            }
        }
    }
    return false;
}

bool CBGBitmap::CreateFromBitmap(BITMAP &bmS)
{
    if (bmS.bmWidth&&bmS.bmHeight&&(bmS.bmBitsPixel==32))
    {
        if (m_hObject)
        {
            BITMAP bmD;
            GetBitmap(&bmD);
            //Only create bitmap if neccesary...
            if ( (bmS.bmWidth!=bmD.bmWidth)||(bmS.bmHeight!=bmD.bmHeight)||(bmS.bmBitsPixel!=bmD.bmBitsPixel) )
            {
                DeleteObject();
                CreateBitmapIndirect(&bmS);
                SetBitmapDimension(bmS.bmWidth, bmS.bmHeight);
            }
        }
        else
        {
            CreateBitmapIndirect(&bmS);
            SetBitmapDimension(bmS.bmWidth, bmS.bmHeight);
        }
    }
    return (m_hObject!=NULL);
}

bool CBGBitmap::CopyTransparent(int iTransparencyPercent, COLORREF crTransparent, CBitmap &bmSrc)
{
    if (bmSrc.m_hObject)
    {
        BITMAP bmS;
        bmSrc.GetBitmap(&bmS);
        if ( CreateFromBitmap(bmS) )
        {
            //Precalculate values...
            BYTE rLevel[256];
            BYTE gLevel[256];
            BYTE bLevel[256];
            for(int bg=0; bg<256; ++bg)
            {
                rLevel[bg] = GetRValue(crTransparent)+MulDiv(bg-GetRValue(crTransparent),iTransparencyPercent,100);
                gLevel[bg] = GetGValue(crTransparent)+MulDiv(bg-GetGValue(crTransparent),iTransparencyPercent,100);
                bLevel[bg] = GetBValue(crTransparent)+MulDiv(bg-GetBValue(crTransparent),iTransparencyPercent,100);
            }
            //Apply to bitmap...
            BYTE *pS = new BYTE[bmS.bmWidthBytes*bmS.bmHeight];
            bmSrc.GetBitmapBits(bmS.bmWidthBytes*bmS.bmHeight, pS);
            for(int row=0; row<bmS.bmHeight; ++row)
            {
                DWORD dwS = row*bmS.bmWidthBytes;
                for(int col=0; col<bmS.bmWidth; ++col)
                {
                    pS[dwS+0] = bLevel[pS[dwS+0]];
                    pS[dwS+1] = gLevel[pS[dwS+1]];
                    pS[dwS+2] = rLevel[pS[dwS+2]];
                    dwS+=4;
                }
            }
            SetBitmapBits(bmS.bmWidthBytes*bmS.bmHeight, pS);
            delete pS;
            return true;
        }
    }
    return false;
}

////////////////////////////////////////////////////////////////////////////////////////////////
//Bitmap drawing functions

bool CBGBitmap::DrawPattern(CDC *pDC, CRect &rc)
{
    if (pDC)
    {
        if (m_hObject)
        {
            CBrush br;  br.CreatePatternBrush(this);
            CBrush *pBr = pDC->SelectObject(&br);
            pDC->FillRect(&rc, &br);
            pDC->SelectObject(pBr);
        }
        else
        {
            pDC->FillSolidRect(&rc, GetSysColor(COLOR_BTNFACE));
        }
        return true;
    }
    return false;
}

bool CBGBitmap::DrawPatternParentOrg(CDC *pDC, CWnd *pWnd, CRect &rcDraw)
{
    CWnd* pParent = pWnd->GetParent();
    //Important for use with toolbar controls: Get the current window origin!
    //Since a toolbar moves the window origin to button origin, while drawing a button!
    CPoint pt = pDC->GetWindowOrg(); 
    pWnd->MapWindowPoints(pParent, &pt, 1);
    pt = pDC->SetBrushOrg(-pt.x,-pt.y);
    bool result = DrawPattern(pDC, rcDraw);
    pDC->SetBrushOrg(pt.x,pt.y);
    return result;
}

bool CBGBitmap::DrawPatternParentOrg(CDC *pDC, CWnd *pWnd)
{
    CRect rc;
    pWnd->GetClientRect(&rc);
    return DrawPatternParentOrg(pDC, pWnd, rc);
}
