
#include "stdafx.h"
#include "CLMgrEventSink.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



IMPLEMENT_DYNCREATE(CClientChat, CCmdTarget)


CClientChat::CClientChat()
{
	EnableAutomation();
    m_dwChatMessageReaderId = -1;
    m_dwChatMessageWriterId = -1;
    m_pfnCallback = NULL;
    m_dwCookie    = 0;
}



CClientChat::~CClientChat()
{
}



void CClientChat::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// delete the object. Add additional cleanup required for your
	// object before calling the base class.
	CCmdTarget::OnFinalRelease();
}



BEGIN_MESSAGE_MAP(CClientChat, CCmdTarget)
	//{{AFX_MSG_MAP(CClientChat)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



BEGIN_DISPATCH_MAP(CClientChat, CCmdTarget)
	//{{AFX_DISPATCH_MAP(CClientChat)
	DISP_FUNCTION(CClientChat, "DispOnLineMgrNotification", DispOnLineMgrNotification, VT_EMPTY, VTS_I4 VTS_I4)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()



//  Note: we add support for IID_IClientLineMgrEventsDisp to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.
//  {f8e55365-4c00-11d3-80bc-00105a653379}

const IID IID_IClientLineMgrEventsDisp={0xf8e55365, 0x4c00, 0x11d3, { 0x80, 0xbc, 0x00, 0x10, 0x5a, 0x65, 0x33, 0x79 }};



BEGIN_INTERFACE_MAP(CClientChat, CCmdTarget)
	INTERFACE_PART(CClientChat, IID_IClientLineMgrEventsDisp, Dispatch)
END_INTERFACE_MAP()



/////////////////////////////////////////////////////////////////////////////
// CClientChat message handlers

HRESULT CClientChat::Register()
{
    m_dwChatMessageReaderId = -1;
    //create line manager object on local machine
    HRESULT hr = m_IClientLineMgrPub.CoCreateInstance(CLSID_ClientLineMgr,NULL,CLSCTX_LOCAL_SERVER);
    if(SUCCEEDED(hr))
    {
        hr = m_IClientLineMgrPub.QueryInterface(&m_IClientLineMgrPub5);
        if(SUCCEEDED(hr))
        {
            hr = m_IClientLineMgrPub5->PubRegisterChatMessageReader(&m_dwChatMessageReaderId);
        }
    }
    return hr;
}

HRESULT CClientChat::GetLogonInfo(CString &strServer, CString &strUser)
{
    CComPtr< IClientLineMgrPub2 > IClientLineMgrPub2;
    HRESULT hr = m_IClientLineMgrPub.QueryInterface(&IClientLineMgrPub2);
    if(SUCCEEDED(hr))
    {
        CComBSTR psServer;
        CComBSTR psUser;
        hr = IClientLineMgrPub2->PubGetLogonInfo(&psServer, &psUser);
        if(SUCCEEDED(hr))
        {
            strServer = psServer;
            strUser = psUser;
        }
    }
    return hr;
}

HRESULT CClientChat::Unregister()
{
    HRESULT hr = S_OK;
    if( (m_dwChatMessageReaderId!=-1)&&m_IClientLineMgrPub5 )
    {
        hr = m_IClientLineMgrPub5->PubUnRegisterChatMessageReader(m_dwChatMessageReaderId);            
        if(SUCCEEDED(hr))
        {
            m_dwChatMessageReaderId = -1;
        }
    }
    return hr;
}

HRESULT CClientChat::Advise()
{
    m_adviseCookie          = 0;
    HRESULT hr = S_OK;
    if (m_IClientLineMgrPub)
    {
        //connect our event sink to the line managers connection point (event source)
        CComPtr< IConnectionPointContainer > icpc;
        hr = m_IClientLineMgrPub.QueryInterface(&icpc);
        if(SUCCEEDED(hr))
        {
            CComPtr< IConnectionPoint > icp;
            //Find connection point for events you're interested in.
            hr = icpc->FindConnectionPoint(IID_IClientLineMgrEventsDisp,&icp);
            if(SUCCEEDED(hr))
            {
                //Get the IUnknown interface of your event implementation.
                LPUNKNOWN pUnk=GetInterface(&IID_IUnknown);
                if(pUnk!=NULL)
                {
                    //connect to event source
                    hr = icp->Advise(pUnk, &m_adviseCookie);
                }
            }
        }
    }
    return hr;
}

HRESULT CClientChat::Unadvise()
{
    HRESULT hr = S_OK;
    if (m_IClientLineMgrPub)
    {
        //connect our event sink to the line managers connection point (event source)
        CComPtr< IConnectionPointContainer > icpc;
        hr = m_IClientLineMgrPub.QueryInterface(&icpc);
        if(SUCCEEDED(hr))
        {
            CComPtr< IConnectionPoint > icp;
            //Find connection point for events you're interested in.
            hr = icpc->FindConnectionPoint(IID_IClientLineMgrEventsDisp,&icp);
            if(SUCCEEDED(hr))
            {
                hr = icp->Unadvise(m_adviseCookie);
                if(SUCCEEDED(hr))
                {
                    m_adviseCookie = 0;
                }
            }
        }
    }
    //m_IClientLineMgrPub.Release();
    //m_IClientLineMgrPub5.Release();
    return hr;
}

HRESULT CClientChat::GetSpeedDial(int Index, CString &strName)
{
    HRESULT hr = E_FAIL;
    if (m_IClientLineMgrPub)
    {
        CComBSTR bstrName;
        hr = m_IClientLineMgrPub->PubGetSpeedDialName(Index, &bstrName);
        if( (hr==S_OK) && bstrName.Length() )
        {
            strName = bstrName;
            return S_OK;
        }
    }
    return hr;
}        

HRESULT CClientChat::GetSpeedDials(CArray<CString, CString&> &arNames)
{
    HRESULT hr = S_OK;
    if (m_IClientLineMgrPub)
    {
        long lNumberOfSpeedDials=0;
        hr = m_IClientLineMgrPub->PubGetNumberOfSpeedDials(&lNumberOfSpeedDials);
        if(hr==S_OK)
        {
            for(long l=0; l<lNumberOfSpeedDials; l++)
            {
                CString strName;
                if (SUCCEEDED(GetSpeedDial(l, strName)))
                {
                    if (!strName.Trim().IsEmpty())
                    {
                        arNames.Add(strName);
                    }
                }
            }
        }
    }
    return hr;
}        

void CClientChat::SetCallbackFunction(void (*pfnCallback)(DWORD dwCookie, long msg, long param), DWORD dwCookie)
{
    m_pfnCallback = pfnCallback;
    m_dwCookie    = dwCookie;
}

void CClientChat::DispOnLineMgrNotification(long msg, long param) 
{
    if (m_pfnCallback)
    {
        m_pfnCallback(m_dwCookie, msg, param);
    }
}

HRESULT CClientChat::SendMessage(CString &strName, CByteArray &arInput)
{
    HRESULT hr = E_INVALIDARG;
    if( (m_dwChatMessageReaderId!=-1)&&m_IClientLineMgrPub5)
    {
        if (!strName.IsEmpty())
        {
            CComBSTR bstrName = strName;
            PubCLMgrChatMsg ChatMsg;
            ChatMsg.m_dwPeerIpAddress=0;
            ChatMsg.m_sPeerName=bstrName;
            ChatMsg.m_sMessageText=SysAllocStringByteLen( (LPCSTR)arInput.GetData(), arInput.GetSize() );
            ChatMsg.m_ulMessageId=m_dwChatMessageWriterId++;
            hr = m_IClientLineMgrPub5->PubSendChatMessage(ChatMsg);
        }
    }
    return hr;
}

HRESULT CClientChat::ReadMessage(CString &strBuddy, CByteArray &arOutput)
{
    if( (m_dwChatMessageReaderId!=-1)&&m_IClientLineMgrPub5)
    {
        PubCLMgrChatMsg ChatMsg;
        ChatMsg.m_dwPeerIpAddress=0;
        ChatMsg.m_sPeerName=NULL;
        ChatMsg.m_sMessageText=NULL;
        ChatMsg.m_ulMessageId=0;

        HRESULT hr = m_IClientLineMgrPub5->PubReadChatMessage(m_dwChatMessageReaderId, &ChatMsg);
        if(hr==S_OK)
        {
            PubCLMgrChatMsgAck ChatMsgAck;
            ChatMsgAck.m_ulMessageId=ChatMsg.m_ulMessageId;
            ChatMsgAck.m_sPeerName=ChatMsg.m_sPeerName;
	        ChatMsgAck.m_dwPeerIpAddress=ChatMsg.m_dwPeerIpAddress;
            hr = m_IClientLineMgrPub5->PubAcknowledgeChatMessage(ChatMsgAck);

            if(ChatMsg.m_sPeerName)
            {
                strBuddy=ChatMsg.m_sPeerName;
                SysFreeString(ChatMsg.m_sPeerName);
                ChatMsg.m_sPeerName=NULL;
                ChatMsgAck.m_sPeerName=NULL;
            }
            if(ChatMsg.m_sMessageText)
            {
                UINT TextLen = SysStringByteLen(ChatMsg.m_sMessageText);
                arOutput.SetSize(TextLen);
                memcpy(arOutput.GetData(), (BYTE *)ChatMsg.m_sMessageText, TextLen);
                SysFreeString(ChatMsg.m_sMessageText);
                ChatMsg.m_sMessageText=NULL;
            }
        }
        return hr;
    }
    return E_INVALIDARG;
}
