#include "stdafx.h"
#include "Emoticon.h"
#include "VisualStylesXP.h"
#include "BGBitmap.h"
#include "comdef.h"
#include "oledlg.h"
#include "ChatEditCtrl.h"

#define     CECHAR      5

extern CVisualStylesXP VisualStylesXP;

///////////////////////////////////////////////////////////////////////////////////////////

CIconListCtrl::CIconListCtrl()
{
    m_iSpacing = 0;
}

CIconListCtrl::~CIconListCtrl()
{
}

bool CIconListCtrl::CreateIcons(UINT nBitmapID, int cx, COLORREF crMask, int iSpacing)
{
    if (m_IconList.Create(nBitmapID, cx, 1, crMask))
    {
        m_iSpacing = iSpacing;
        SetImageList(&m_IconList,LVSIL_NORMAL);    
        SetIconSpacing(iSpacing, iSpacing);
        SetExtendedStyle(GetExtendedStyle()|LVS_EX_GRIDLINES);
        for(int i=0; i<m_IconList.GetImageCount(); ++i)
        {
            InsertItem(i,L"",i);
        }
        return true;
    }
    return false;
}

BEGIN_MESSAGE_MAP(CIconListCtrl, CRichEditCtrl)
    ON_WM_LBUTTONDOWN()
END_MESSAGE_MAP()

//Since selection of desired item via mouse doesn't work correctly, 
//let's make our own selection method...
void CIconListCtrl::OnLButtonDown(UINT nFlags, CPoint point)
{
    CRect rcView;
    GetViewRect(&rcView);
    CPoint pt((point.x-rcView.left)/m_iSpacing,(point.y-rcView.top)/m_iSpacing);
    int nIconsPerRow = rcView.Width()/m_iSpacing;
    int Index = pt.y*nIconsPerRow+pt.x;
    if ( (pt.x<nIconsPerRow)&&(Index>=0)&&(Index<GetItemCount()) )
    {
        NMLISTVIEW lv;
        lv.hdr.code = NM_CLICK;
        lv.hdr.hwndFrom = m_hWnd;
        lv.hdr.idFrom = ::GetDlgCtrlID(m_hWnd);
        lv.iItem = Index;
        lv.iSubItem = 0;
        lv.lParam = 0;
        lv.ptAction = POINT();
        lv.uChanged = 0;
        lv.uNewState = 0;
        lv.uOldState = 0;
        GetParent()->SendMessage(WM_NOTIFY, lv.hdr.idFrom, (LPARAM)&lv);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////

CCharFormat::CCharFormat(   bool bBold,
                            bool bItalic,
                            bool bUnderline,
                            int  nTwipsHeight,
                            COLORREF crText,
                            CString &strFaceName)
{
    cbSize = sizeof(CHARFORMAT);
    dwMask = CFM_COLOR|CFM_FACE|CFM_BOLD|CFM_ITALIC|CFM_SIZE|CFM_UNDERLINE;
    dwEffects = (bBold?CFE_BOLD:0)|(bItalic?CFE_ITALIC:0)|(bUnderline?CFE_UNDERLINE:0)|CFE_PROTECTED;
    yHeight = nTwipsHeight;
    yOffset = 0;
    crTextColor = crText;
    bCharSet = 0;
    bPitchAndFamily = 0;
    wcscpy_s(szFaceName, strFaceName);
    wWeight = 0;
    sSpacing = 0;
    crBackColor = 0;
    lcid = 0;
    dwReserved = 0;
    sStyle = 0;
    wKerning = 0;
    bUnderlineType = 0;
    bAnimation = 0;
    bRevAuthor = 0;
}

CCharFormat::CCharFormat(   bool bBold,
                            bool bItalic,
                            bool bUnderline,
                            int  nTwipsHeight,
                            COLORREF crText,
                            CString &strFaceName,
                            COLORREF crBG)
{
    cbSize = sizeof(CHARFORMAT2);
    dwMask = CFM_COLOR|CFM_FACE|CFM_BOLD|CFM_ITALIC|CFM_SIZE|CFM_UNDERLINE|CFM_BACKCOLOR;
    dwEffects = (bBold?CFE_BOLD:0)|(bItalic?CFE_ITALIC:0)|(bUnderline?CFE_UNDERLINE:0)|CFE_PROTECTED;
    yHeight = nTwipsHeight;
    yOffset = 0;
    crTextColor = crText;
    bCharSet = 0;
    bPitchAndFamily = 0;
    wcscpy_s(szFaceName, strFaceName);
    wWeight = 0;
    sSpacing = 0;
    crBackColor = crBG;
    lcid = 0;
    dwReserved = 0;
    sStyle = 0;
    wKerning = 0;
    bUnderlineType = 0;
    bAnimation = 0;
    bRevAuthor = 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////

CChatEditCtrl::CChatEditCtrl(CBitmap &bm) : m_bm(bm)
{
    m_pRichEditOle = NULL;
    m_crTrans = RGB(255,255,255);
    m_crTransDefault = m_crTrans;
    m_iTrans = 5;
}

CChatEditCtrl::~CChatEditCtrl()
{
    m_pRichEditOle->Release();
}


BEGIN_MESSAGE_MAP(CChatEditCtrl, CRichEditCtrl)
    ON_WM_NCCALCSIZE()
    ON_WM_NCPAINT()
    ON_WM_ENABLE()
    ON_WM_ERASEBKGND()
    ON_WM_KEYDOWN()
END_MESSAGE_MAP()


void CChatEditCtrl::RedrawFrame()
{
    m_pRichEditOle = GetIRichEditOle();
    //Force the control to recalc size of client rect...
    SetWindowPos(NULL,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER|SWP_FRAMECHANGED);
}

bool CChatEditCtrl::CalcClientRect(RECT &rcClient)
{
	HTHEME hTheme = VisualStylesXP.OpenThemeData (m_hWnd, L"edit");
	if (hTheme!=NULL)
	{
        CRect rcContent;
        CDC *pDC = GetDC();
        VisualStylesXP.GetThemeBackgroundContentRect(hTheme, *pDC, EP_EDITTEXT, ETS_NORMAL, &rcClient, &rcContent);
        rcClient = rcContent;
        ReleaseDC(pDC);
        VisualStylesXP.CloseThemeData(hTheme);
        return true;
    }
    return false;
}

void CChatEditCtrl::OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS* lpncsp)
{
    if (CalcClientRect(lpncsp->rgrc[0]))
    {
        DWORD dwStyle = GetStyle();
        DWORD dwExStyle = GetExStyle();
        if (dwStyle&WS_BORDER)
            ModifyStyle(WS_BORDER,0);
        if (dwExStyle&WS_EX_CLIENTEDGE)
            ModifyStyleEx(WS_EX_CLIENTEDGE,0);
        CRichEditCtrl::OnNcCalcSize(bCalcValidRects, lpncsp);
    }
    else
    {
        CRichEditCtrl::OnNcCalcSize(bCalcValidRects, lpncsp);
    }
}

int CChatEditCtrl::GetState()
{
    int nState = ETS_NORMAL;
    if(!IsWindowEnabled())
        nState = ETS_DISABLED;
    else if(GetOptions()&ECO_READONLY)
        nState = ETS_READONLY;
    return nState;
}

void CChatEditCtrl::OnNcPaint()
{
	HTHEME hTheme = VisualStylesXP.OpenThemeData (m_hWnd, L"edit");
	if (hTheme!=NULL)
	{
        CRichEditCtrl::OnNcPaint();
        RECT rcBorder;
        GetWindowRect(&rcBorder);
        rcBorder.right-=rcBorder.left; 
        rcBorder.bottom-=rcBorder.top;
        rcBorder.left = rcBorder.top = 0;
        //Draw the border of the edit box
        CDC *pDC = GetWindowDC();
        if(VisualStylesXP.IsThemeBackgroundPartiallyTransparent(hTheme, EP_EDITTEXT, ETS_NORMAL))
        {
            VisualStylesXP.DrawThemeParentBackground(m_hWnd, *pDC, &rcBorder);
        }
        DTBGOPTS opt;
        opt.dwSize = sizeof(DTBGOPTS);
        opt.dwFlags = DTBG_OMITCONTENT;
        VisualStylesXP.DrawThemeBackgroundEx(hTheme, *pDC, EP_EDITTEXT, GetState(), &rcBorder, &opt);
        ReleaseDC(pDC);
        VisualStylesXP.CloseThemeData(hTheme);
    }
    else
    {
        CRichEditCtrl::OnNcPaint();
    }
}

void CChatEditCtrl::OnEnable(BOOL bEnable)
{
    CRichEditCtrl::OnEnable(bEnable);
    RedrawWindow(NULL, NULL, RDW_INVALIDATE|RDW_NOCHILDREN|RDW_UPDATENOW|RDW_FRAME);
}

BOOL CChatEditCtrl::OnEraseBkgnd(CDC* pDC)
{
    CBGBitmap bm;
    bm.CopyTransparent(m_iTrans, m_crTrans, m_bm);
    bm.DrawPatternParentOrg(pDC, this);
    return true;//CRichEditCtrl::OnEraseBkgnd(pDC);
}

BOOL CChatEditCtrl::OnWndMsg(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	if((message==WM_THEMECHANGED)||(message==WM_STYLECHANGED))
	{
        RedrawWindow(NULL, NULL, RDW_INVALIDATE|RDW_NOCHILDREN|RDW_UPDATENOW|RDW_FRAME);
        return true;
	}
    return CRichEditCtrl::OnWndMsg(message, wParam, lParam, pResult);
}

void CChatEditCtrl::InsertString(CString &strText, CHARFORMAT2 &cf)
{
    CByteArray ar; 
    ar.SetSize(strText.GetLength()*sizeof(TCHAR));
    memcpy(ar.GetData(),strText.GetBuffer(),strText.GetLength()*sizeof(TCHAR));
    SendMessage(EM_SETCHARFORMAT,SCF_SELECTION,(LPARAM)&cf);
    int format = 0;
    if (sizeof(TCHAR)==sizeof(WCHAR))
        format = SF_UNICODE;
    SetContent(ar,SF_TEXT|SFF_SELECTION|format);
}

///////////////////////////////////////////////////////////////////////////////////////

DWORD CALLBACK CChatEditCtrl::FileStreamOutCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
   CFile* pFile = (CFile*)dwCookie;
   pFile->Write(pbBuff, cb);
   *pcb = cb;
   return 0;
}

DWORD CALLBACK CChatEditCtrl::FileStreamInCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
   CFile* pFile = (CFile*)dwCookie;
   *pcb = pFile->Read(pbBuff, cb);
   return 0;
}

bool CChatEditCtrl::GetContentFile(CString &strFile, int nFormat)
{
   CFile f;
   if (f.Open(strFile,CFile::modeCreate|CFile::modeWrite))
   {
       EDITSTREAM es;
       es.dwCookie = (DWORD)&f;
       es.pfnCallback = FileStreamOutCallback; 
       StreamOut(nFormat, es);
       return true;
   }
   return false;
}

bool CChatEditCtrl::SetContentFile(CString &strFile, int nFormat)
{
   CFile f;
   if (f.Open(strFile,CFile::modeRead))
   {
       EDITSTREAM es;
       es.dwCookie = (DWORD)&f;
       es.pfnCallback = FileStreamInCallback; 
       StreamIn(nFormat, es);
       return true;
   }
   return false;
}

void CChatEditCtrl::GetContent(CByteArray &arContent, int nFormat)
{
    arContent.RemoveAll();
    CStreamContent sc(arContent, true);
    StreamOut(nFormat, sc);
}

void CChatEditCtrl::SetContent(CByteArray &arContent, int nFormat)
{
    CStreamContent sc(arContent, false);
	StreamIn(nFormat, sc);
}

void CChatEditCtrl::GetObjectsContent(CByteArray &arContent)
{
	if (m_pRichEditOle)
	{
        //First store object count, position and #icon to ByteArray
	    REOBJECT re;
        int cObj = m_pRichEditOle->GetObjectCount();
        int size = CECHAR+cObj*(CECHAR+CECHAR);
        arContent.RemoveAll();
        arContent.SetSize(size);
        BYTE *pDst = arContent.GetData();
        if (pDst)
        {
            char szValue[8];
            sprintf_s(szValue,"%*d",CECHAR,cObj);
            memcpy(pDst, szValue, CECHAR); pDst+=CECHAR;
	        for (int i=0; i<cObj; ++i) 
	        {
	            ZeroMemory(&re, sizeof(REOBJECT));
		        re.cbStruct  = sizeof(REOBJECT);
		        HRESULT hr = m_pRichEditOle->GetObject(i,&re,REO_GETOBJ_NO_INTERFACES);
		        if(SUCCEEDED(hr))
		        {
                    sprintf_s(szValue, "%*d",CECHAR,re.dwUser);
                    memcpy(pDst, szValue, CECHAR); pDst+=CECHAR;
                    sprintf_s(szValue, "%*d",CECHAR,re.cp);
                    memcpy(pDst, szValue, CECHAR); pDst+=CECHAR;
		        }
	        }
            //Second store rtf content without ole objects*/
            CStreamContent sc(arContent, true, size);
            StreamOut(SF_RTF|SF_RTFNOOBJS, sc); //Replaces OLE items with spaces
        }
	}
}

void CChatEditCtrl::SetObjectsContent(CByteArray &arContent, CImageList &EmoticonList)
{
	if (m_pRichEditOle)
	{
        SetRedraw(false); //ensure flickerfree drawing
        long s,e;
        GetSel(s,e); //get current selection
        //Load icon count from ByteArray and calculate size
        BYTE *pSrc = arContent.GetData();
        if (pSrc&&arContent.GetSize()>4)
        {
            char szValue[8];
            strncpy_s(szValue, (const char *)pSrc, CECHAR); pSrc+=CECHAR; 
            szValue[CECHAR] = 0;
            int cObj = atoi(szValue);
            int size = CECHAR+cObj*(CECHAR+CECHAR);
            //Load rtf content
            CStreamContent sc(arContent, false, size);
	        StreamIn(SF_RTF|SFF_SELECTION, sc);
            //Load icon position and #icon from ByteArray and insert icons
            long ms,me;
            GetSel(ms,me);
		    for (int i=0; i<cObj; ++i) 
		    {
                strncpy_s(szValue, (const char *)pSrc, CECHAR); pSrc+=CECHAR;
                szValue[CECHAR] = 0;
                int nIcon = atoi(szValue);
                strncpy_s(szValue, (const char *)pSrc, CECHAR); pSrc+=CECHAR;
                szValue[CECHAR] = 0;
                int cp = atoi(szValue);
                cp+=e;
                SetSel(cp,cp+1);
                ReplaceSel(L"",false); //Delete space
                //Insert Icon instead
		        InsertEmoticon(EmoticonList, nIcon, cp);
		    }
            SetSel(ms,me); //Restore selection
        }
        SendMessage(WM_VSCROLL, SB_BOTTOM, 0);
        SetRedraw(true);
        InvalidateRect(NULL);
        UpdateWindow();
	}
}

void CChatEditCtrl::InsertEmoticon(CImageList &EmoticonList, WORD nEmoticon, LONG cp)
{
    CComPtr<IOleClientSite> pClientSite = NULL;
    HRESULT hr = m_pRichEditOle->GetClientSite((IOleClientSite **)&pClientSite);
    if ( SUCCEEDED(hr)&&pClientSite )
    {
	    CComObject<CEmoticon>* pItem = NULL;
        HRESULT hr = CComObject<CEmoticon>::CreateInstance(&pItem);
        if ( SUCCEEDED(hr)&&pItem )
        {
            pItem->SetIconNumber(EmoticonList, nEmoticon);
            CComPtr<IOleObject> *pObject = NULL;
            hr = pItem->QueryInterface(IID_IOleObject,(void**)&pObject);
            if ( SUCCEEDED(hr)&&pObject )
            {
	            OleSetContainedObject((IOleObject *)pObject, FALSE);
	            REOBJECT reobject;
	            ZeroMemory(&reobject, sizeof(REOBJECT));
	            reobject.cbStruct = sizeof(REOBJECT);	
	            reobject.clsid = CLSID_Emoticon;
	            reobject.cp = cp;
	            reobject.dvaspect = DVASPECT_CONTENT;
	            reobject.dwFlags = REO_BELOWBASELINE;
	            reobject.dwUser = nEmoticon;
	            reobject.poleobj = (IOleObject *)pObject;
	            reobject.polesite = pClientSite;
	            reobject.pstg = NULL;
	            hr = m_pRichEditOle->InsertObject(&reobject);
            }
        }    		
    }
}

void CChatEditCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    if ((nChar==VK_RETURN)&&(GetAsyncKeyState(VK_CONTROL)<0))
    {
        ::SendMessage(::GetParent(m_hWnd),WM_COMMAND,IDC_BUTTONSEND,(LPARAM)m_hWnd);
    }
    else
    {
        CRichEditCtrl::OnKeyDown(nChar, nRepCnt, nFlags);
    }
}

void CChatEditCtrl::SetCurToEnd()
{
    long l = GetTextLength();
    SetSel(l,l);
}

///////////////////////////////////////////////////////////////////////////////////////////

CChatThreadCtrl::CChatThreadCtrl(CBitmap &bm) : CChatEditCtrl(bm), 
                                                m_cfHeadline(true,false,false,160,RGB(0,0,0),CString(L"Arial"),RGB(255,255,0))
{
    m_cfHeadlineDefault = m_cfHeadline;
}

CCharFormat &CChatThreadCtrl::GetCFHeadline()
{
    return m_cfHeadline;
}

CCharFormat &CChatThreadCtrl::GetCFHeadlineDefault()
{
    return m_cfHeadlineDefault;
}

void CChatThreadCtrl::InsertHeadline(CString &strText)
{
    SetCurToEnd();
    InsertString(strText, m_cfHeadline);
    LONG h = m_cfHeadline.yHeight;
    m_cfHeadline.yHeight = MulDiv(m_cfHeadline.yHeight, 5, 12);
    InsertString(CString("\n"), m_cfHeadline);
    m_cfHeadline.yHeight = h;
}
