#pragma once

#include "atlsimpstr.h"
#include "resource.h"

///////////////////////////////////////////////////////////////////////////////////////////

class CIconListCtrl : public CListCtrl
{
private:
    CImageList  m_IconList;
    int         m_iSpacing;

public:
    CIconListCtrl();
    ~CIconListCtrl();
    CImageList &GetIconList()
    {
        return m_IconList;
    }
    bool CreateIcons(UINT nBitmapID, int cx, COLORREF crMask, int iSpacing);

protected:
	DECLARE_MESSAGE_MAP()

public:
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
};

///////////////////////////////////////////////////////////////////////////////////////////

class CCharFormat : public CHARFORMAT2 // : public CHARFORMAT
{
public:
    CCharFormat::CCharFormat(   bool bBold,
                                bool bItalic,
                                bool bUnderline,
                                int  nTwipsHeight,
                                COLORREF crText,
                                CString &strFaceName);
    CCharFormat::CCharFormat(   bool bBold,
                                bool bItalic,
                                bool bUnderline,
                                int  nTwipsHeight,
                                COLORREF crText,
                                CString &strFaceName,
                                COLORREF crBG);
    CCharFormat::CCharFormat()
    {
        memset(this,0,sizeof(this));
    }
};

///////////////////////////////////////////////////////////////////////////////////////////

class CChatEditCtrl : public CRichEditCtrl
{
private:
    //nested class for streaming purposes...
    class CStreamContent : public EDITSTREAM
    {
        private:
            CByteArray  &m_arContent;
            DWORD       m_dwPos;

        public:
            CStreamContent(CByteArray &arContent, bool bStreamOut, int dwPos=0);
            LONG CStreamContent::StreamOut(const BYTE *pDst, LONG cBytes);
            static DWORD CALLBACK EditStreamOutCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb);
            LONG CStreamContent::StreamIn(BYTE *pSrc, LONG cBytes);
            static DWORD CALLBACK EditStreamInCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb);
    };

private:
    int             GetState();
    bool            CalcClientRect(RECT &rcClient);
    IRichEditOle    *m_pRichEditOle;
    CBitmap         &m_bm;

public:
    COLORREF        m_crTrans;
    COLORREF        m_crTransDefault;
    int             m_iTrans;
    //CHARFORMAT2     m_cfDefault;

public:
	CChatEditCtrl(CBitmap &bm);
	virtual ~CChatEditCtrl();
    void    RedrawFrame();
    void InsertString(CString &strText, CHARFORMAT2 &cf);
    static DWORD CALLBACK FileStreamOutCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb);
    static DWORD CALLBACK FileStreamInCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb);
    bool GetContentFile(CString &strFile, int nFormat);
    bool SetContentFile(CString &strFile, int nFormat);
    void GetContent(CByteArray &arContent, int nFormat);
    void SetContent(CByteArray &arContent, int nFormat);
    void GetObjectsContent(CByteArray &arContent);
    void SetObjectsContent(CByteArray &arContent, CImageList &EmoticonList);
    void ClearContent()
    {
        SetSel(0, -1);
        Clear();
    }
    //void RestoreDefaultCharFormat();
    void InsertEmoticon(CImageList &EmoticonList, WORD nEmoticon, LONG cp=REO_CP_SELECTION);
    void SetCurToEnd();

protected:
	DECLARE_MESSAGE_MAP()

protected:
    virtual BOOL OnWndMsg(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pResult);

public:
    afx_msg void OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS* lpncsp);
    afx_msg void OnNcPaint();
    afx_msg void OnEnable(BOOL bEnable);
    afx_msg BOOL OnEraseBkgnd(CDC* pDC);
    afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
};


///////////////////////////////////////////////////////////////////////////////////////////

class CChatThreadCtrl : public CChatEditCtrl
{
private:
    CCharFormat m_cfHeadline;
    CCharFormat m_cfHeadlineDefault;

public:
    CChatThreadCtrl(CBitmap &bm);
    CCharFormat &GetCFHeadline();
    CCharFormat &GetCFHeadlineDefault();
    void InsertHeadline(CString &strText);
};