#include "stdafx.h"
#include "VisualStylesXP.h"
#include <math.h>
#include "ColorCtrl.h"

#define     CURSORSIZE      8

extern CVisualStylesXP VisualStylesXP;

#define crmax(a, b)  (((a) > (b)) ? (a) : (b))
#define crmin(a, b)  (((a) < (b)) ? (a) : (b)) 

///////////////////////////////////////////////////////////////////////////////////////////

CColorCtrl::CColorCtrl()
{
    m_bColorPlane = false;
    m_CursorPoint.x = m_CursorPoint.y = -1;
}

CColorCtrl::~CColorCtrl()
{
}


BEGIN_MESSAGE_MAP(CColorCtrl, CWnd)
    ON_WM_PAINT()
    ON_WM_SETCURSOR()
    ON_WM_NCHITTEST()
    ON_WM_LBUTTONDOWN()
    ON_WM_MOUSEMOVE()
    ON_WM_LBUTTONUP()
    ON_WM_NCCALCSIZE()
    ON_WM_NCPAINT()
END_MESSAGE_MAP()

void CColorCtrl::RedrawFrame()
{
    //Force the control to recalc size of client rect...
    SetWindowPos(NULL,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER|SWP_FRAMECHANGED);
    //Bitmap width must be divisible by 4!
    //For this reason adjust size of color control...
    CRect rcW;
    CRect rcC;
    GetWindowRect(&rcW);
    GetClientRect(&rcC);
    int delta = rcC.right-rcC.right/4*4; 
    SetWindowPos(NULL,0,0,rcW.Width()-delta,rcW.Height(),SWP_NOMOVE|SWP_NOZORDER);
}

bool CColorCtrl::CalcClientRect(RECT &rcClient)
{
	HTHEME hTheme = VisualStylesXP.OpenThemeData (m_hWnd, L"edit");
	if (hTheme!=NULL)
	{
        CRect rcContent;
        CDC *pDC = GetDC();
        VisualStylesXP.GetThemeBackgroundContentRect(hTheme, *pDC, EP_EDITTEXT, ETS_NORMAL, &rcClient, &rcContent);
        rcClient = rcContent;
        ReleaseDC(pDC);
        VisualStylesXP.CloseThemeData(hTheme);
        return true;
    }
    return false;
}

void CColorCtrl::OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS* lpncsp)
{
    if (!CalcClientRect(lpncsp->rgrc[0]))
    {
        lpncsp->rgrc[0].top+=1;
        lpncsp->rgrc[0].left+=1;
        lpncsp->rgrc[0].bottom-=1;
        lpncsp->rgrc[0].right-=1;
    }
}

int CColorCtrl::GetState()
{
    int nState = ETS_NORMAL;
    if(!IsWindowEnabled())
        nState = ETS_DISABLED;
    return nState;
}

void CColorCtrl::OnNcPaint()
{
	HTHEME hTheme = VisualStylesXP.OpenThemeData (m_hWnd, L"edit");
	if (hTheme!=NULL)
	{
        RECT rcBorder;
        GetWindowRect(&rcBorder);
        rcBorder.right-=rcBorder.left; 
        rcBorder.bottom-=rcBorder.top;
        rcBorder.left = rcBorder.top = 0;
        //Draw the border of the edit box
        CDC *pDC = GetWindowDC();
        if(VisualStylesXP.IsThemeBackgroundPartiallyTransparent(hTheme, EP_EDITTEXT, ETS_NORMAL))
        {
            VisualStylesXP.DrawThemeParentBackground(m_hWnd, *pDC, &rcBorder);
        }
        DTBGOPTS opt;
        opt.dwSize = sizeof(DTBGOPTS);
        opt.dwFlags = DTBG_OMITCONTENT;
        VisualStylesXP.DrawThemeBackgroundEx(hTheme, *pDC, EP_EDITTEXT, GetState(), &rcBorder, &opt);
        ReleaseDC(pDC);
        VisualStylesXP.CloseThemeData(hTheme);
    }
    else
    {
        RECT rcBorder;
        GetWindowRect(&rcBorder);
        rcBorder.right-=rcBorder.left; 
        rcBorder.bottom-=rcBorder.top;
        rcBorder.left = rcBorder.top = 0;

        CDC *pDC = GetWindowDC();
        pDC->DrawEdge(&rcBorder, BDR_RAISEDOUTER, BF_RECT);
        ReleaseDC(pDC);
    }
}

//Normalized values  In: r,g,b Out: h,s,v
void CColorCtrl::RGBtoHSV(float r, float g, float b, float& h, float& s, float& v)
{
	float max = crmax(r,crmax(g,b));
    float min = crmin(r,crmin(g,b));
    float delta = max-min;
    if (delta>0.0f)
    {
        if (r==max)
        {
	        h = (g-b)/delta;	// between yellow & magenta
        }
        else if(g==max)
        {
	        h = 2+(b-r)/delta;	// between cyan & yellow
        }
        else
        {
	        h = 4+(r-g)/delta;	// between magenta & cyan
        }
        h/=6.0;			        // normalize
        if(h<0.0f)
        {
	        h+=1.0;
        }
		s = delta/max;
    }
	else 
    {
		h = s = 0.0f;
	}
    v = max;
}

//Normalized values  In: h,s,v Out: r,g,b
void CColorCtrl::HSVtoRGB(float h, float s, float v, float& r, float& g, float& b)
{
	if (s>0.0f)  // achromatic (grey)
    {
	    h*=6.0;			// sector 0 to 5
	    int i = (int)floor(h);
	    float f = h - i;			// factorial part of h
	    float p = v * ( 1 - s );
	    float q = v * ( 1 - s * f );
	    float t = v * ( 1 - s * ( 1 - f ) );
        if (i==0)
        {
		    r = v; g = t; b = p;
        }
        else if (i==1)
        {
		    r = q; g = v; b = p;
        }
        else if (i==2)
        {
		    r = p; g = v; b = t;
        }
        else if (i==3)
        {
		    r = p; g = q; b = v;
        }
        else if (i==4)
        {
		    r = t; g = p; b = v;
        }
        else if (i==5)
        {
		    r = v; g = p; b = q;
        }
	}
    else
    {
		r = g = b = v;
	}
}

void CColorCtrl::DrawCursor(CPoint& point)
{
    CDC *pDC = GetDC();
    if (pDC)
    {
        if ((m_CursorPoint.x>=0)&&(m_CursorPoint.y>=0))
        {
            CSize sz = m_bmp.GetBitmapDimension();
            int OldMode = pDC->SetROP2(R2_NOT);
            if (m_bColorPlane)
            {
                pDC->MoveTo(m_CursorPoint.x,m_CursorPoint.y-CURSORSIZE+1);
                pDC->LineTo(m_CursorPoint.x,m_CursorPoint.y+CURSORSIZE);
                pDC->MoveTo(m_CursorPoint.x+CURSORSIZE-1,m_CursorPoint.y);
                pDC->LineTo(m_CursorPoint.x-CURSORSIZE,m_CursorPoint.y);
            }
            else
            {
                pDC->MoveTo(0,m_CursorPoint.y);
                pDC->LineTo(sz.cx,m_CursorPoint.y);
            }
            pDC->SetROP2(OldMode);
        }
        m_CursorPoint = point;
        RefreshCursor(pDC);
        ReleaseDC(pDC);
    }
}

void CColorCtrl::RefreshCursor(CDC *pDC)
{
    if (pDC&&(m_CursorPoint.x>=0)&&(m_CursorPoint.y>=0))
    {
        CSize sz = m_bmp.GetBitmapDimension();
        int OldMode = pDC->SetROP2(R2_NOT);
        if (m_bColorPlane)
        {
            pDC->MoveTo(m_CursorPoint.x,m_CursorPoint.y-CURSORSIZE+1);
            pDC->LineTo(m_CursorPoint.x,m_CursorPoint.y+CURSORSIZE);
            pDC->MoveTo(m_CursorPoint.x+CURSORSIZE-1,m_CursorPoint.y);
            pDC->LineTo(m_CursorPoint.x-CURSORSIZE,m_CursorPoint.y);
        }
        else
        {
            pDC->MoveTo(0,m_CursorPoint.y);
            pDC->LineTo(sz.cx,m_CursorPoint.y);
        }
        pDC->SetROP2(OldMode);
    }
}

void CColorCtrl::DrawColorPlane()
{
    RedrawFrame();
    m_bColorPlane = true;
    CRect rcClient;
    GetClientRect(&rcClient);
    int cw = rcClient.Width();
    int ch = rcClient.Height();
    BYTE *pRGB = new BYTE[cw*ch*3];
    DWORD dwPos = 0;
    float r, g, b;
    //float hue,sat,val;
    for(int h=0; h<ch; ++h)
    {
        for(int w=0; w<cw; ++w)
        {
            HSVtoRGB(float(w)/(float)(cw-1), float(h)/(float)(ch-1), 1.0, r, g, b);
            //RGBtoHSV(r,g,b,hue,sat,val);
            //HSVtoRGB(hue,sat,val,r,g,b);
            pRGB[dwPos++] = (BYTE)(b*255.0);
            pRGB[dwPos++] = (BYTE)(g*255.0);
            pRGB[dwPos++] = (BYTE)(r*255.0);
        }
    }
    CreateBitmap(cw,ch,pRGB);
    delete pRGB;
}

void CColorCtrl::DrawGradient()
{
    RedrawFrame();
    m_bColorPlane = false;
    CRect rcClient;
    GetClientRect(&rcClient);
    int cw = rcClient.Width();
    int ch = rcClient.Height();
    BYTE *pRGB = new BYTE[cw*ch*3];
    DWORD dwPos = 0;
    for(int h=0; h<ch; ++h)
    {
        float v = (float)(h)/(float)(ch-1)*255.0f;
        for(int w=0; w<cw; ++w)
        {
            pRGB[dwPos++] = (BYTE)v;
            pRGB[dwPos++] = (BYTE)v;
            pRGB[dwPos++] = (BYTE)v;
        }
    }
    CreateBitmap(cw,ch,pRGB);
    delete pRGB;
}

void CColorCtrl::CreateBitmap(int w, int h, BYTE *pRGB)
{
    BITMAPINFO bmi;
    //bmi.bmiColors[0] = 0;
    bmi.bmiHeader.biBitCount = 24;
    bmi.bmiHeader.biClrImportant = 0;
    bmi.bmiHeader.biClrUsed = 0;
    bmi.bmiHeader.biCompression = BI_RGB;
    bmi.bmiHeader.biHeight = h; // bottom-up DIB
    bmi.bmiHeader.biPlanes = 1;
    bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bmi.bmiHeader.biSizeImage = w*h*3;
    bmi.bmiHeader.biWidth = w;
    bmi.bmiHeader.biXPelsPerMeter = 0;
    bmi.bmiHeader.biYPelsPerMeter = 0;

    CDC *pDC = GetDC();
    HBITMAP hBmp = ::CreateDIBitmap(*pDC, &bmi.bmiHeader, CBM_INIT, pRGB, &bmi, DIB_RGB_COLORS);
    m_bmp.Attach(hBmp);
    m_bmp.SetBitmapDimension(w,h);
    ReleaseDC(pDC);
}

void CColorCtrl::NotifyParent()
{
    CWnd *pP = GetParent();
    if (pP)
    {
        pP->SendMessage(WM_COMMAND,MAKELONG(GetDlgCtrlID(),0));
    }
}

void CColorCtrl::SetColor(COLORREF cr)
{
    float nr = (float)GetRValue(cr)/255.0f;
    float ng = (float)GetGValue(cr)/255.0f;
    float nb = (float)GetBValue(cr)/255.0f;
    float h,s,v;
    RGBtoHSV(nr,ng,nb,h,s,v);

    CSize sz = m_bmp.GetBitmapDimension();
    if (m_bColorPlane)
    {
        m_CursorPoint.x = (int)(h*(float)sz.cx);
        m_CursorPoint.y = (int)((1.0f-s)*(float)sz.cy);
    }
    else
    {
        m_CursorPoint.x = 0;
        m_CursorPoint.y = (int)((1.0f-v)*(float)(sz.cy-1));
    }
    Invalidate();
}

COLORREF CColorCtrl::GetColor(COLORREF cr)
{
    if ((m_CursorPoint.x>=0)&&(m_CursorPoint.y>=0))
    {
        float nr = (float)GetRValue(cr)/255.0f;
        float ng = (float)GetGValue(cr)/255.0f;
        float nb = (float)GetBValue(cr)/255.0f;
        float h,s,v;
        RGBtoHSV(nr,ng,nb,h,s,v);

        CSize sz = m_bmp.GetBitmapDimension();
        if (m_bColorPlane)
        {
            h = (float)m_CursorPoint.x/(float)sz.cx;
            s = 1.0f-(float)m_CursorPoint.y/(float)sz.cy;
        }
        else
        {
            v = 1.0f-(float)m_CursorPoint.y/(float)(sz.cy-1);
        }
        HSVtoRGB(h,s,v,nr,ng,nb);
        return RGB((BYTE)(nr*255.0),(BYTE)(ng*255.0),(BYTE)(nb*255.0));
    }
    return cr;
}

void CColorCtrl::OnPaint()
{
    CPaintDC dc(this); // device context for painting
    if (m_bmp.m_hObject)
    {
        CSize sz = m_bmp.GetBitmapDimension();
	    CDC memDC;
	    memDC.CreateCompatibleDC(&dc);
	    CBitmap *pOld = memDC.SelectObject(&m_bmp);
        dc.BitBlt(0, 0, sz.cx, sz.cy, &memDC, 0, 0, SRCCOPY);
	    memDC.SelectObject(pOld);
	    memDC.DeleteDC();
        RefreshCursor(&dc);
    }
}

BOOL CColorCtrl::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
    if (GetAsyncKeyState(VK_LBUTTON)<0)
        SetCursor(LoadCursor(NULL, L""));
    return CWnd::OnSetCursor(pWnd, nHitTest, message);
}

LRESULT CColorCtrl::OnNcHitTest(CPoint point)
{
    return HTCLIENT;//CWnd::OnNcHitTest(point);
}

void CColorCtrl::OnLButtonDown(UINT nFlags, CPoint point)
{
    CWnd::OnLButtonDown(nFlags, point);
    CSize sz = m_bmp.GetBitmapDimension();
    CRect rc(0,0,sz.cx,sz.cy);
    ClientToScreen(&rc);
    ClipCursor(&rc);
    DrawCursor(point);
}

void CColorCtrl::OnMouseMove(UINT nFlags, CPoint point)
{
    CWnd::OnMouseMove(nFlags, point);
    if (GetAsyncKeyState(VK_LBUTTON)<0)
    {
        DrawCursor(point);
        NotifyParent();
    }
}

void CColorCtrl::OnLButtonUp(UINT nFlags, CPoint point)
{
    CWnd::OnLButtonUp(nFlags, point);
    ClipCursor(NULL);
    DrawCursor(point);
    NotifyParent();
}
