#include "stdafx.h"
#include "VisualStylesXP.h"
#include "BGBitmap.h"
#include "ColorPicker.h"

extern CVisualStylesXP VisualStylesXP;

void CColorPicker::CColorItem::Draw(CDC *pDC, HTHEME hTheme, int nState, int iInflate)
{
    if (pDC)
    {
        CRect rcColor = m_rcColor;
        rcColor.InflateRect(iInflate,iInflate);
        pDC->FillSolidRect(&rcColor,m_cr);
	    if (hTheme!=NULL)
	    {
            DTBGOPTS opt;
            opt.dwSize = sizeof(DTBGOPTS);
            opt.dwFlags = DTBG_OMITCONTENT;
            VisualStylesXP.DrawThemeBackgroundEx(hTheme, *pDC, EP_EDITTEXT, nState, &rcColor, &opt);
        }
        else
        {
            pDC->DrawEdge(&rcColor, BDR_RAISEDOUTER, BF_RECT);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////////////////

CColorPicker::CColorPicker(CBGBitmap &bm) : m_bm(bm)
{
    m_ptPrev.SetPoint(-1,-1);
    m_ptSel.SetPoint(-1,-1);
}

CColorPicker::~CColorPicker()
{
}


BEGIN_MESSAGE_MAP(CColorPicker, CWnd)
    ON_WM_PAINT()
    ON_WM_SETCURSOR()
    ON_WM_NCHITTEST()
    ON_WM_LBUTTONDOWN()
    ON_WM_MOUSEMOVE()
    ON_WM_LBUTTONUP()
    ON_WM_NCCALCSIZE()
    ON_WM_NCPAINT()
    ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

void CColorPicker::OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS* lpncsp)
{
    //Clientrect==Windowrect nothing to adjust...
}

void CColorPicker::OnNcPaint()
{
}

bool CColorPicker::Check(int c, int r)
{
    return (c>=0)&&(c<m_cols)&&(r>=0)&&(r<m_rows);
}

int CColorPicker::GetState()
{
    int nState = ETS_NORMAL;
    if(!IsWindowEnabled())
        nState = ETS_DISABLED;
    return nState;
}

void CColorPicker::NotifyParent()
{
    CWnd *pP = GetParent();
    if (pP)
    {
        pP->SendMessage(WM_COMMAND,MAKELONG(GetDlgCtrlID(),0));
    }
}

COLORREF CColorPicker::GetColor()
{
    if (Check(m_ptSel.x,m_ptSel.y))
    {
        return m_ColorItems[m_ptSel.x][m_ptSel.y].m_cr;
    }
    return 0;
}

void CColorPicker::SetColorItem(int c, int r, COLORREF cr)
{
    if (Check(c,r))
    {
        m_ColorItems[c][r].m_cr = cr;
    }
}

bool CColorPicker::SetSel(COLORREF cr)
{
    for(int col=0; col<m_cols; ++col)
    {
        for(int row=0; row<m_rows; ++row)
        {
            if (m_ColorItems[col][row].m_cr==cr)
            {
                SetSel(col,row);
                return true;
            }
        }
    }
    return false;
}

void CColorPicker::SetSel(int c, int r)
{
    if (Check(c,r))
    {
        CDC *pDC = GetDC();
        HTHEME hTheme = VisualStylesXP.OpenThemeData (m_hWnd, L"edit");
        if (Check(m_ptSel.x,m_ptSel.y))
        {
            m_bm.DrawPatternParentOrg(pDC, this, m_ColorItems[m_ptSel.x][m_ptSel.y].m_rcItem);
            m_ColorItems[m_ptSel.x][m_ptSel.y].Draw(pDC, hTheme, GetState());
        }
        m_ColorItems[c][r].Draw(pDC, hTheme, GetState(), 2);
        if (hTheme)
        {
            VisualStylesXP.CloseThemeData(hTheme);
        }
        ReleaseDC(pDC);
        m_ptPrev.x = -1;
        m_ptPrev.y = -1;
        m_ptSel.x = c;
        m_ptSel.y = r;
    }
}

bool CColorPicker::ColorItemFromPoint(CPoint &pt, int &c, int &r)
{
    for(int col=0; col<m_cols; ++col)
    {
        for(int row=0; row<m_rows; ++row)
        {
            if (m_ColorItems[col][row].m_rcItem.PtInRect(pt))
            {
                c = col;
                r = row;
                return true;
            }
        }
    }
    return false;
}

void CColorPicker::PreSubclassWindow()
{
    CWnd::PreSubclassWindow();
}

void CColorPicker::CalculateColorItems(int Saturation)
{
    CRect rc;
    GetClientRect(&rc);

    int Width = rc.Width()/m_cols;
    int Height = rc.Height()/m_rows;
    CRect rcSpace;
    rcSpace.left    = m_space;
    rcSpace.right   = m_space;
    rcSpace.top     = m_space;
    rcSpace.bottom  = m_space;
    if (Height>Width)
    {
        rcSpace.top = rcSpace.left+(Height-Width)/2;
        rcSpace.bottom = rcSpace.top;
    }
    else 
    {
        rcSpace.left  = rcSpace.top+(Width-Height)/2;
        rcSpace.right = rcSpace.left;
    }

    int sat = 255-Saturation;
    //          gry,red,yel,gre,cya,blu,mag
    WORD r[] = {sat,255,255,sat,sat,sat,255};
    WORD g[] = {sat,sat,255,255,255,sat,sat};
    WORD b[] = {sat,sat,sat,sat,255,255,255};

    //gray levels
    for(int row=0; row<m_rows; ++row)
    {
        m_ColorItems[0][row].m_rcItem.left   = 0;
        m_ColorItems[0][row].m_rcItem.top    = rc.Height()*row/m_rows;
        m_ColorItems[0][row].m_rcItem.right  = Width;
        m_ColorItems[0][row].m_rcItem.bottom = m_ColorItems[0][row].m_rcItem.top+Height;

        m_ColorItems[0][row].m_rcColor       = m_ColorItems[0][row].m_rcItem;
        m_ColorItems[0][row].m_rcColor.left  +=rcSpace.left;
        m_ColorItems[0][row].m_rcColor.top   +=rcSpace.top;
        m_ColorItems[0][row].m_rcColor.right -=rcSpace.right;
        m_ColorItems[0][row].m_rcColor.bottom-=rcSpace.bottom;

        BYTE v = 255*row/(m_rows-1);
        m_ColorItems[0][row].m_cr = RGB(v,v,v);
    }
    //colors
    for(int col=1; col<m_cols; ++col)
    {
        //saturation levels
        for(int row=0; row<m_rows; ++row)
        {
            m_ColorItems[col][row].m_rcItem.left   = rc.Width()*col/m_cols;
            m_ColorItems[col][row].m_rcItem.top    = rc.Height()*row/m_rows;
            m_ColorItems[col][row].m_rcItem.right  = m_ColorItems[col][row].m_rcItem.left+Width;
            m_ColorItems[col][row].m_rcItem.bottom = m_ColorItems[col][row].m_rcItem.top+Height;

            m_ColorItems[col][row].m_rcColor       = m_ColorItems[col][row].m_rcItem;
            m_ColorItems[col][row].m_rcColor.left  +=rcSpace.left;
            m_ColorItems[col][row].m_rcColor.top   +=rcSpace.top;
            m_ColorItems[col][row].m_rcColor.right -=rcSpace.right;
            m_ColorItems[col][row].m_rcColor.bottom-=rcSpace.bottom;
            //Interpolation between white and destination color
            m_ColorItems[col][row].m_cr = RGB(  r[col]+(255-r[col])*row/m_rows,
                                                g[col]+(255-g[col])*row/m_rows,
                                                b[col]+(255-b[col])*row/m_rows);
        }
    }
}

BOOL CColorPicker::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
    return CWnd::OnSetCursor(pWnd, nHitTest, message);
}

LRESULT CColorPicker::OnNcHitTest(CPoint point)
{
    return HTCLIENT;//CWnd::OnNcHitTest(point);
}

void CColorPicker::OnMouseMove(UINT nFlags, CPoint point)
{
    int c;
    int r;
    if ( ColorItemFromPoint(point, c, r) )
    {
        CDC *pDC = GetDC();
	    HTHEME hTheme = VisualStylesXP.OpenThemeData (m_hWnd, L"edit");
        if ((m_ptPrev.x!=c)||(m_ptPrev.y!=r))
        {
            if (Check(m_ptPrev.x,m_ptPrev.y))
            {
                if ((m_ptSel.x!=m_ptPrev.x)||(m_ptSel.y!=m_ptPrev.y))
                {
                    m_bm.DrawPatternParentOrg(pDC, this, m_ColorItems[m_ptPrev.x][m_ptPrev.y].m_rcItem);
                    m_ColorItems[m_ptPrev.x][m_ptPrev.y].Draw(pDC, hTheme, GetState());
                }
            }
            else
            {
                TRACKMOUSEEVENT tme;
                tme.cbSize = sizeof(TRACKMOUSEEVENT);
                tme.dwFlags = TME_LEAVE;
                tme.dwHoverTime = 0;
                tme.hwndTrack = m_hWnd;
                TrackMouseEvent(&tme);
            }
            m_ptPrev.x = c;
            m_ptPrev.y = r;
        }
        if ((m_ptSel.x!=c)||(m_ptSel.y!=r))
        {
            m_ColorItems[c][r].Draw(pDC, hTheme, GetState(), 1);
        }
        if (hTheme)
        {
            VisualStylesXP.CloseThemeData(hTheme);
        }
        ReleaseDC(pDC);
    }
    CWnd::OnMouseMove(nFlags, point);
}

BOOL CColorPicker::OnWndMsg(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
    if (message==WM_MOUSELEAVE)
    {
        CDC *pDC = GetDC();
        HTHEME hTheme = VisualStylesXP.OpenThemeData (m_hWnd, L"edit");
        if (Check(m_ptPrev.x,m_ptPrev.y))
        {
            if ((m_ptSel.x!=m_ptPrev.x)||(m_ptSel.y!=m_ptPrev.y))
            {
                m_bm.DrawPatternParentOrg(pDC, this, m_ColorItems[m_ptPrev.x][m_ptPrev.y].m_rcItem);
                m_ColorItems[m_ptPrev.x][m_ptPrev.y].Draw(pDC, hTheme, GetState());
            }
        }
        m_ptPrev.x = -1;
        m_ptPrev.y = -1;
        if (hTheme)
        {
            VisualStylesXP.CloseThemeData(hTheme);
        }
        ReleaseDC(pDC);
    }
    return CWnd::OnWndMsg(message, wParam, lParam, pResult);
}

void CColorPicker::OnLButtonDown(UINT nFlags, CPoint point)
{
    CWnd::OnLButtonDown(nFlags, point);
    int c;
    int r;
    if ( ColorItemFromPoint(point, c, r) )
    {
        SetSel(c,r);
    }
}

void CColorPicker::OnLButtonUp(UINT nFlags, CPoint point)
{
    CWnd::OnLButtonUp(nFlags, point);
    if (Check(m_ptSel.x,m_ptSel.y))
    {
        NotifyParent();
    }
}

void CColorPicker::OnPaint()
{
    CPaintDC dc(this); // device context for painting
    CRect rc;
    GetClientRect(&rc);

    CBitmap bm;
    if ( bm.CreateCompatibleBitmap(&dc, rc.Width(), rc.Height()) )
    {
	    CDC memDC;
	    if (memDC.CreateCompatibleDC(&dc))
        {
	        CBitmap* pOldBitmap = memDC.SelectObject(&bm);
            m_bm.DrawPatternParentOrg(&memDC, this, rc);
	        HTHEME hTheme = VisualStylesXP.OpenThemeData (m_hWnd, L"edit");
            //Colors
            for(int col=0; col<m_cols; ++col)
            {
                for(int row=0; row<m_rows; ++row)
                {
                    int inflate = 0;
                    if ((m_ptSel.x==col)&&(m_ptSel.y==row))
                    {
                        inflate = 2;
                    }
                    else if ((m_ptPrev.x==col)&&(m_ptPrev.y==row))
                    {
                        inflate = 1;
                    }
                    m_ColorItems[col][row].Draw(&memDC, hTheme, GetState(), inflate);
                }
            }
            if (hTheme)
            {
                VisualStylesXP.CloseThemeData(hTheme);
            }
            dc.BitBlt(0,0,rc.Width(),rc.Height(),&memDC,0,0,SRCCOPY);
	        memDC.SelectObject(pOldBitmap);
        }
    }
}

BOOL CColorPicker::OnEraseBkgnd(CDC* pDC)
{
    return true;//CWnd::OnEraseBkgnd(pDC);
}
