#include "stdafx.h"
#include "BGBitmap.h"
#include "FontCtrl.h"

///////////////////////////////////////////////////////////////////////////////////////////

CFontSizeSlider::CFontSizeSlider(CBGBitmap &bm) : m_bm(bm)
{
    m_TwipsMin = 0;
    m_TwipsMax = 0;
    m_nTwips = 0;
}

CFontSizeSlider::~CFontSizeSlider()
{
}

void CFontSizeSlider::SetTwipsRange(int TwipsMin, int TwipsMax)
{
    m_TwipsMin  = TwipsMin;
    m_TwipsMax  = TwipsMax;
    m_nTwips    = TwipsMax-TwipsMin;
    CSliderCtrl::SetRange(0, m_nTwips);
}

void CFontSizeSlider::SetTwips(int nTwips)
{
    nTwips = m_nTwips-nTwips;
    int pos = nTwips-m_TwipsMin;
    if (pos<0)
        pos = 0;
    else if (pos>m_nTwips)
        pos = m_nTwips;
    SetPos(pos);
}

int CFontSizeSlider::GetTwips()
{
    int nTwips = GetPos()+m_TwipsMin;
    nTwips = m_nTwips-nTwips;
    if (nTwips<m_TwipsMin)
        nTwips = m_TwipsMin;
    else if (nTwips>m_TwipsMax)
        nTwips = m_TwipsMax;
    return nTwips;
}

BEGIN_MESSAGE_MAP(CFontSizeSlider, CSliderCtrl)
    ON_WM_ERASEBKGND()
    ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, &CFontSizeSlider::OnNMCustomdraw)
END_MESSAGE_MAP()


BOOL CFontSizeSlider::OnEraseBkgnd(CDC* pDC)
{
    return true;//CSliderCtrl::OnEraseBkgnd(pDC);
}

void CFontSizeSlider::OnNMCustomdraw(NMHDR *pNMHDR, LRESULT *pResult)
{
    LPNMCUSTOMDRAW pNMCD = reinterpret_cast<LPNMCUSTOMDRAW>(pNMHDR);
    if (pNMCD->dwDrawStage==CDDS_PREPAINT)
    {
        *pResult = CDRF_NOTIFYITEMDRAW;
    }
    else if (pNMCD->dwDrawStage==CDDS_ITEMPREPAINT)
    {
        if (pNMCD->dwItemSpec==TBCD_CHANNEL)
        {
            CDC *pDC = CDC::FromHandle(pNMCD->hdc);
            m_bm.DrawPatternParentOrg(pDC, this);
            pDC->Draw3dRect(&pNMCD->rc, GetSysColor(COLOR_3DSHADOW), GetSysColor(COLOR_3DLIGHT));
            if ((pNMCD->uItemState&CDIS_FOCUS)||
                (::GetFocus()==m_hWnd))
            {
                CRect rc;
                GetClientRect(&rc);
                pDC->DrawFocusRect(&rc);
            }
            *pResult = CDRF_SKIPDEFAULT;
        }
        else if (pNMCD->dwItemSpec==TBCD_THUMB)
        {
            *pResult = CDRF_DODEFAULT;
        }
        else if (pNMCD->dwItemSpec==TBCD_TICS)
        {
            *pResult = CDRF_SKIPDEFAULT;
        }
        else *pResult = CDRF_DODEFAULT;
    } else *pResult = CDRF_DODEFAULT;
}

///////////////////////////////////////////////////////////////////////////////////////////

CFontListbox::CFontListbox()
{

}

CFontListbox::~CFontListbox()
{
}

int CALLBACK CFontListbox::EnumFontFamProc(ENUMLOGFONT *lpelf, NEWTEXTMETRIC *lpntm,
					                        int nFontType, LPARAM lParam)
{
	CFontListbox* pLb = (CFontListbox*)lParam;
    
    //if (lpelf->elfLogFont.lfCharSet==ANSI_CHARSET)
    //{
        int Index = pLb->AddString(lpelf->elfLogFont.lfFaceName);
        pLb->SetItemData(Index, (nFontType==TRUETYPE_FONTTYPE)?1:0);
    //}
	return 1;		// 1 to continue enumeration
}

void CFontListbox::GetFontNames()
{
    SetTabStops(40);
    CDC *pDC = GetDC();
	::EnumFontFamilies(*pDC,NULL,(FONTENUMPROC)EnumFontFamProc,(LPARAM )this);
    ReleaseDC(pDC);
}

bool CFontListbox::SetSelFont(CString &strFont)
{
    return (SelectString(-1,strFont)!=LB_ERR);
}

CString CFontListbox::GetSelFont()
{
    CString strFont;
    int i = GetCurSel();
    if (i!=LB_ERR)
    {
        GetText(i,strFont);
    }
    return strFont;
}

void CFontListbox::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
   CDC dc; dc.Attach(lpDrawItemStruct->hDC);

   // Save these value to restore them when done drawing.
   COLORREF crOldTextColor = dc.GetTextColor();
   COLORREF crOldBkColor = dc.GetBkColor();

   // If this item is selected, set the background color 
   // and the text color to appropriate values. Also, erase
   // rect by filling it with the background color.
   if ((lpDrawItemStruct->itemAction|ODA_SELECT)&&(lpDrawItemStruct->itemState&ODS_SELECTED))
   {
      dc.SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
      dc.SetBkColor(::GetSysColor(COLOR_HIGHLIGHT));
      dc.FillSolidRect(&lpDrawItemStruct->rcItem, ::GetSysColor(COLOR_HIGHLIGHT));
   }
   else
   {
      dc.FillSolidRect(&lpDrawItemStruct->rcItem, crOldBkColor);
   }
   CRect rcText = lpDrawItemStruct->rcItem;
   rcText.left+=2; rcText.right-=2;
   CString strText;
   GetText(lpDrawItemStruct->itemID, strText);
   dc.DrawText(strText,&rcText,DT_LEFT|DT_SINGLELINE|DT_VCENTER);

   if (GetItemData(lpDrawItemStruct->itemID))
   {
       dc.DrawText(CString(" TT"),&rcText,DT_RIGHT|DT_SINGLELINE|DT_VCENTER);
   }
   // If this item has the focus, draw a red frame around the
   // item's rect.
   if ((lpDrawItemStruct->itemAction|ODA_FOCUS)&&(lpDrawItemStruct->itemState&ODS_FOCUS))
   {
       dc.DrawFocusRect(&lpDrawItemStruct->rcItem);
   }
   // Reset the background color and the text color back to their
   // original values.
   dc.SetTextColor(crOldTextColor);
   dc.SetBkColor(crOldBkColor);

   dc.Detach();
}

BEGIN_MESSAGE_MAP(CFontListbox, CListBox)
END_MESSAGE_MAP()


///////////////////////////////////////////////////////////////////////////////////////////

CFontSizeListbox::CFontSizeListbox()
{

}

CFontSizeListbox::~CFontSizeListbox()
{
}

void CFontSizeListbox::PreSubclassWindow()
{
    CListBox::PreSubclassWindow();
    CString strFontsize;
    int ptFontSize[] = { 5,6,7,8,9,10,11,12,13,14,15,16,18,20,24,28,32,40 };
    for(int f=0; f<sizeof(ptFontSize)/sizeof(int); ++f)
    {
        strFontsize.Format(L"%d",ptFontSize[f]);
        AddString(strFontsize);
    }
}

void CFontSizeListbox::SetPt(int nPt)
{
    CString strFontsize;
    strFontsize.Format(L"%d",nPt);
    int f = FindStringExact(-1,strFontsize);
    if (f>=0)
        SetCurSel(f);
    else SetCurSel(0);
}

int CFontSizeListbox::GetPt()
{
    int s = GetCurSel();
    if (s>=0)
    {
        CString strFontSize;
        GetText(s,strFontSize);
        return _ttoi(strFontSize);
    }
    return 0;
}

BEGIN_MESSAGE_MAP(CFontSizeListbox, CListBox)
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////////////////
