#include "stdafx.h"
#include "resource.h"
#include "BGBitmap.h"
#include "ColorCtrl.h"
#include "ColorPicker.h"
#include "SettingsDlg.h"


CSettingsDlg::CSettingsDlg(CSettingsData &sd, CWnd* pParent /*=NULL*/) : m_ColorPicker(m_bm), CDialog(CSettingsDlg::IDD, pParent)
{
    m_Initial = sd;
    m_Current = sd;
    m_nColorItem = 0;
}

CSettingsDlg::~CSettingsDlg()
{
}

void CSettingsDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_EDITBITMAP, m_EditBitmap);
    for(int i=IDC_RADIOCOLORFIRST; i<=IDC_RADIOCOLORLAST; ++i)
    {
        DDX_Control(pDX, i, m_RadioColor[i-IDC_RADIOCOLORFIRST]);
    }
    DDX_Control(pDX, IDC_COLORPICKER, m_ColorPicker);
    DDX_Control(pDX, IDC_SLIDERTRANSPARENCY, m_SliderTransparency);
    DDX_Control(pDX, IDC_STATICTRANS, m_TransLabel);
    DDX_Control(pDX, IDC_STATICTRANSVALUE, m_TransValue);
    DDX_Control(pDX, IDC_COLORPLANE, m_ColorPlane);
    DDX_Control(pDX, IDC_COLORVALUE, m_ColorValue);
}


BEGIN_MESSAGE_MAP(CSettingsDlg, CDialog)
    ON_BN_CLICKED(IDC_BUTTONBITMAP, &CSettingsDlg::OnBnClickedButtonBitmap)
    ON_EN_KILLFOCUS(IDC_EDITBITMAP, &CSettingsDlg::OnEnKillfocusEditbitmap)
    ON_COMMAND_RANGE(IDC_RADIOCOLORFIRST, IDC_RADIOCOLORLAST, OnRadioColor)
    ON_BN_CLICKED(IDC_COLORPICKER, &CSettingsDlg::OnClickedColorpicker)
    ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_SLIDERTRANSPARENCY, &CSettingsDlg::OnNMReleasedcaptureSlidertransparency)
    ON_WM_HSCROLL()
    ON_BN_CLICKED(IDC_DEFAULTCOLOR, &CSettingsDlg::OnBnClickedDefaultcolor)
    ON_COMMAND(IDC_COLORPLANE, &CSettingsDlg::OnColorPlane)
    ON_COMMAND(IDC_COLORVALUE, &CSettingsDlg::OnColorValue)
    ON_BN_CLICKED(IDC_RADIOCOLORFIRST, &CSettingsDlg::OnBnClickedRadiocolorfirst)
END_MESSAGE_MAP()

void CSettingsDlg::NotifyParent()
{
    m_Current.SettingsChanged();
}

BOOL CSettingsDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    m_ColorPlane.DrawColorPlane();
    m_ColorValue.DrawGradient();
    m_ColorPicker.CalculateColorItems(255);
    m_EditBitmap.SetWindowText(m_Current.m_strBitmap);
    m_SliderTransparency.SetRange(0,100);
    m_RadioColor[0].SetCheck(1);
    SetColorItem(0);

    return TRUE;  // return TRUE unless you set the focus to a control
}

void CSettingsDlg::OnBnClickedButtonBitmap()
{
    TCHAR szFilters[]= L"BMP Files (*.bmp)|*.bmp|";
    CFileDialog fileDlg (TRUE, L"bmp", L"*.bmp", OFN_FILEMUSTEXIST|OFN_HIDEREADONLY, szFilters, this);
    if(fileDlg.DoModal()==IDOK)
    {
        m_Current.m_strBitmap = fileDlg.GetPathName();
        m_EditBitmap.SetWindowText(m_Current.m_strBitmap);
        NotifyParent();
    }
}

void CSettingsDlg::OnEnKillfocusEditbitmap()
{
    m_EditBitmap.GetWindowText(m_Current.m_strBitmap);
    NotifyParent();
}

void CSettingsDlg::SetColorItem(int nColorItem)
{
    m_nColorItem = nColorItem;
    if (m_Current.m_Transparency[m_nColorItem]>=0)
    {
        m_ColorPicker.ShowWindow(SW_HIDE);
        m_ColorPlane.ShowWindow(SW_SHOW);
        m_ColorValue.ShowWindow(SW_SHOW);
        m_TransLabel.EnableWindow(true);
        m_TransValue.EnableWindow(true);
        m_SliderTransparency.EnableWindow(true);
        m_SliderTransparency.SetPos(m_Current.m_Transparency[m_nColorItem]);
        m_ColorPlane.SetColor(m_Current.m_cr[m_nColorItem]);
        m_ColorValue.SetColor(m_Current.m_cr[m_nColorItem]);
    }
    else
    {
        m_ColorPicker.ShowWindow(SW_SHOW);
        m_ColorPlane.ShowWindow(SW_HIDE);
        m_ColorValue.ShowWindow(SW_HIDE);
        m_TransLabel.EnableWindow(false);
        m_TransValue.EnableWindow(false);
        m_SliderTransparency.EnableWindow(false);
        m_SliderTransparency.SetPos(0);
        if (!m_ColorPicker.SetSel(m_Current.m_cr[m_nColorItem]))
        {
            m_ColorPicker.SetColorItem(0,0,m_Current.m_cr[m_nColorItem]);
            m_ColorPicker.SetSel(0,0);
        }
    }
    TCHAR sz[100];
    _stprintf_s(sz, L"%d", m_SliderTransparency.GetPos());
    m_TransValue.SetWindowText(sz);
}

void CSettingsDlg::OnBnClickedDefaultcolor()
{
    m_Current.m_cr[m_nColorItem] = m_Current.m_crDefault[m_nColorItem];
    if (m_Current.m_Transparency[m_nColorItem]>=0)
    {
        m_ColorPlane.SetColor(m_Current.m_cr[m_nColorItem]);
        m_ColorValue.SetColor(m_Current.m_cr[m_nColorItem]);
    }
    else
    {
        if (!m_ColorPicker.SetSel(m_Current.m_cr[m_nColorItem]))
        {
            m_ColorPicker.SetColorItem(0,0,m_Current.m_cr[m_nColorItem]);
            m_ColorPicker.SetSel(0,0);
        }
    }
    NotifyParent();
}

void CSettingsDlg::OnRadioColor(UINT nID)
{
    SetColorItem(nID-IDC_RADIOCOLORFIRST);
}

void CSettingsDlg::OnClickedColorpicker()
{
    m_Current.m_cr[m_nColorItem] = m_ColorPicker.GetColor();
    NotifyParent();
}

void CSettingsDlg::OnColorPlane()
{
    m_Current.m_cr[m_nColorItem] = m_ColorPlane.GetColor(m_ColorValue.GetColor());
    NotifyParent();
}

void CSettingsDlg::OnColorValue()
{
    m_Current.m_cr[m_nColorItem] = m_ColorPlane.GetColor(m_ColorValue.GetColor());
    NotifyParent();
}

void CSettingsDlg::OnNMReleasedcaptureSlidertransparency(NMHDR *pNMHDR, LRESULT *pResult)
{
    m_Current.m_Transparency[m_nColorItem] = m_SliderTransparency.GetPos();
    NotifyParent();
    *pResult = 0;
}

void CSettingsDlg::OnCancel()
{
    m_Current = m_Initial;
    CDialog::OnCancel();
}

void CSettingsDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
    m_Current.m_Transparency[m_nColorItem] = m_SliderTransparency.GetPos();
    TCHAR sz[100];
    _stprintf_s(sz, L"%d", m_Current.m_Transparency[m_nColorItem]);
    m_TransValue.SetWindowText(sz);
    CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
    NotifyParent();
}

void CSettingsDlg::OnBnClickedRadiocolorfirst()
{
    // TODO: Add your control notification handler code here
}
