#include "stdafx.h"
#include "resource.h"
#include "VisualStylesXP.h"
#include "BGBitmap.h"
#include "TransparentCtrl.h"

extern CVisualStylesXP VisualStylesXP;

///////////////////////////////////////////////////////////////////////////////////////////

CTransparentCheckbox::CTransparentCheckbox(CBGBitmap &bm) : m_bm(bm)
{
}

CTransparentCheckbox::~CTransparentCheckbox()
{
}

BEGIN_MESSAGE_MAP(CTransparentCheckbox, CButton)
    ON_WM_ERASEBKGND()
    ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, &CTransparentCheckbox::OnNMCustomdraw)
END_MESSAGE_MAP()

void CTransparentCheckbox::OnNMCustomdraw(NMHDR *pNMHDR, LRESULT *pResult)
{
    *pResult = CDRF_DODEFAULT;

    LPNMCUSTOMDRAW pNMCD = reinterpret_cast<LPNMCUSTOMDRAW>(pNMHDR);
    if (pNMCD->dwDrawStage==CDDS_PREPAINT)
    {
        *pResult = CDRF_NOTIFYITEMDRAW;
    }
    else if (pNMCD->dwDrawStage==CDDS_PREERASE)
    {
        m_bm.DrawPatternParentOrg(CDC::FromHandle(pNMCD->hdc), this);
    }
}

BOOL CTransparentCheckbox::OnEraseBkgnd(CDC* pDC)
{
    return true;//CSliderCtrl::OnEraseBkgnd(pDC);
}

///////////////////////////////////////////////////////////////////////////////////////////

CTransparentStatic::CTransparentStatic(CBGBitmap &bm) : m_bm(bm)
{
}

BEGIN_MESSAGE_MAP(CTransparentStatic, CStatic)
    ON_WM_CTLCOLOR_REFLECT()
    ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

HBRUSH CTransparentStatic::CtlColor(CDC* pDC, UINT nCtlColor)
{
   pDC->SetBkMode(TRANSPARENT);
   return (HBRUSH)GetStockObject(NULL_BRUSH);
}

BOOL CTransparentStatic::OnEraseBkgnd(CDC* pDC)
{
    m_bm.DrawPatternParentOrg(pDC, this);
    return true;
}

void CTransparentStatic::SetWindowText(CString &strText)
{
    CDC *pDC = GetDC();
    m_bm.DrawPatternParentOrg(pDC, this);
    ReleaseDC(pDC);
    CStatic::SetWindowText(strText);
}

///////////////////////////////////////////////////////////////////////////////////////////

CTransparentToolbar::CTransparentToolbar(CBGBitmap &bm) : m_bm(bm)
{
}

CTransparentToolbar::~CTransparentToolbar()
{
}

int CTransparentToolbar::Create(CWnd *pParent, int iID)
{
    if ( !CToolBar::CreateEx(pParent,TBSTYLE_FLAT|TBSTYLE_TRANSPARENT|TBSTYLE_CUSTOMERASE,
                             WS_CHILD|WS_VISIBLE|CBRS_TOP|CBRS_TOOLTIPS,
                             &CRect(0,0,0,0),iID)||
         !LoadToolBar(iID) )
    {
        TRACE0("Failed to Create Dialog Toolbar\n");
        return false;
    }
    ModifyStyle(0,TBSTYLE_LIST);
    GetToolBarCtrl().SetExtendedStyle(TBSTYLE_EX_DRAWDDARROWS|TBSTYLE_EX_MIXEDBUTTONS);
    m_bmTB.LoadBitmap(IDB_TOOLBAR);
    m_bmTBM.LoadBitmap(IDB_TOOLBARMASK);

    SetButtonText(0,L"Save");
	SetButtonStyle(0,GetButtonStyle(0)| BTNS_SHOWTEXT);

    SetButtonText(1,L"Clear");
	SetButtonStyle(1,GetButtonStyle(1)| BTNS_SHOWTEXT);

    SetButtonText(2,L"Settings");
	SetButtonStyle(2,GetButtonStyle(2)| BTNS_SHOWTEXT);

    DWORD dwSize = GetToolBarCtrl().GetButtonSize();
    return HIWORD(dwSize);
}

BEGIN_MESSAGE_MAP(CTransparentToolbar, CToolBar)
    ON_WM_ERASEBKGND()
    ON_WM_NCCALCSIZE()
    ON_WM_NCPAINT()
    ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, &CTransparentToolbar::OnNMCustomdraw)
    ON_WM_PAINT()
END_MESSAGE_MAP()

BOOL CTransparentToolbar::OnEraseBkgnd(CDC* pDC)
{
    return true;//CToolBar::OnEraseBkgnd(pDC);
}

void CTransparentToolbar::OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS* lpncsp)
{
    //Do nothing since client rc==window rc
}

void CTransparentToolbar::OnNcPaint()
{
    // TODO: Add your message handler code here
    // Do not call CToolBar::OnNcPaint() for painting messages
}

void CTransparentToolbar::OnNMCustomdraw(NMHDR *pNMHDR, LRESULT *pResult)
{
    *pResult = CDRF_SKIPDEFAULT;
}

COLORREF CTransparentToolbar::GetState(int Index, int HotItem, int &state)
{
    COLORREF cr = GetSysColor(COLOR_WINDOW);
    state = GetToolBarCtrl().GetState(GetItemID(Index));
    int level = 0;
    if (state&TBSTATE_PRESSED)
        level = 2;
    else if ((state&TBSTATE_CHECKED)&&(Index==HotItem))
        level = 3;
    else if (state&TBSTATE_CHECKED)
        level = 2;
    else if (Index==HotItem)
        level = 4;
    if (level)
    {
        level+=12;
        return RGB(GetRValue(cr)*level/16,GetGValue(cr)*level/16,GetBValue(cr)*level/16);
    }
    return 0;
}

void CTransparentToolbar::OnPaint()
{
    CPaintDC dc(this); // device context for painting
    CRect rcClip;
    CRect rcItem;
    CRect rcTemp;
    dc.GetClipBox(&rcClip);
    CFont *pFont = GetFont();
    CFont *pOldFont = dc.SelectObject(pFont);
    int OldBkMode = dc.SetBkMode(TRANSPARENT);
    CDC dcMem;
    dcMem.CreateCompatibleDC(&dc);
    int HotItem = GetToolBarCtrl().GetHotItem();
    int c = GetToolBarCtrl().GetButtonCount();
	HTHEME hTheme = VisualStylesXP.OpenThemeData (m_hWnd, L"EDIT");
    for(int i=0; i<c; ++i)
    {
        GetItemRect(i,&rcItem);
        if (rcTemp.IntersectRect(&rcItem,&rcClip))
        {
            int state = 0;
            COLORREF cr = GetState(i, HotItem, state);
            if (cr)
            {
                dc.FillSolidRect(&rcItem,cr);
	            if (hTheme!=NULL)
	            {
                    DTBGOPTS opt;
                    opt.dwSize = sizeof(DTBGOPTS);
                    opt.dwFlags = DTBG_OMITCONTENT;
                    VisualStylesXP.DrawThemeBackgroundEx(hTheme, dc, EP_EDITTEXT, ETS_NORMAL, &rcItem, &opt);
                }
                else
                {
                    dc.DrawEdge(&rcItem, BDR_RAISEDOUTER, BF_RECT);
                }
            }
            else
            {
                m_bm.DrawPatternParentOrg(&dc, this, CRect(rcItem.left,rcClip.top,rcItem.right,rcClip.bottom));
            }
            CBitmap *pBmM = dcMem.SelectObject(&m_bmTBM);
            dc.BitBlt(rcItem.left+2,(rcItem.Height()-15)/2,16,15,&dcMem,16*i,0,SRCAND);
            dcMem.SelectObject(pBmM);
            if (state&TBSTATE_ENABLED)
            {              
                CBitmap *pBm = dcMem.SelectObject(&m_bmTB);
                dc.BitBlt(rcItem.left+2,(rcItem.Height()-15)/2,16,15,&dcMem,16*i,0,SRCPAINT);
                dcMem.SelectObject(pBm);
                dc.DrawText(GetButtonText(i),&CRect(rcItem.left+20,rcItem.top,rcItem.right-2,rcItem.bottom),DT_SINGLELINE|DT_LEFT|DT_VCENTER|DT_CENTER);
            }
            else
            {
                COLORREF crText = dc.SetTextColor(GetSysColor(COLOR_GRAYTEXT));
                dc.DrawText(GetButtonText(i),&CRect(rcItem.left+20,rcItem.top,rcItem.right-2,rcItem.bottom),DT_SINGLELINE|DT_LEFT|DT_VCENTER|DT_CENTER);
                dc.SetTextColor(crText);
            }
        }
    }
    if (rcClip.right>rcItem.right)
    {
        if (rcClip.left<rcItem.right)
            rcClip.left = rcItem.right;
        m_bm.DrawPatternParentOrg(&dc, this, rcClip);
    }
	if (hTheme!=NULL)
	{
        VisualStylesXP.CloseThemeData(hTheme);
    }
    dc.SetBkMode(OldBkMode);
    dc.SelectObject(pOldFont);
}
