/*---------------------------------------------------------------------------

   Swyx Solutions
   Joseph-von-Fraunhofer-Str. 13a
   44227 Dortmund

  ---------------------------------------------------------------------------

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (C) 2007 Swyx Solutions

  All Rights Reserved.

  ---------------------------------------------------------------------------

     Project:   Swyx Client SDK

      Module:   Visual C++ ATL Client PlugIn sample
 
    Comments:   

----------------------------------------------------------------------------*/

// LdapResolver.cpp : Implementation of CLdapResolver

#include "stdafx.h"
#include "LdapAddIn.h"
#include "CLMgrPubTypes.h"
#include "LdapResolver.h"
#include "githelp.h"
#include <comdef.h>
#include "debugout.h"

//#include <windows.h>
#include <windef.h>
#include <schnlsp.h>

#include <winldap.h>
#include <winber.h>
#include <rpcdce.h>
#include <stdio.h>







HRESULT AddSearchResultToCollection(INameNumberSearchResultCollection* pISRC, CString szName, CString szNumber, CString szDescription)
{
    HRESULT hRetVal = E_FAIL;

    try
    {
        if(pISRC)
        {
            IDispatch* pIDispResult = NULL;
            //create new result item
            hRetVal = pISRC->get_CreateItem(&pIDispResult);
            if(SUCCEEDED(hRetVal) && pIDispResult)
            {                             
                INameNumberSearchResult* pIResult = NULL;

                hRetVal = pIDispResult->QueryInterface(IID_INameNumberSearchResult, (void**)&pIResult);
                if(SUCCEEDED(hRetVal) && pIResult)
                {                                     
                    //set item data (name, number, description)
                    pIResult->put_Name(szName.AllocSysString());
                    pIResult->put_Number(szNumber.AllocSysString());
                    pIResult->put_Description(szDescription.AllocSysString());

                    //commit item to collection
                    hRetVal = pISRC->AddItem(pIDispResult);
                                                            
                    pIResult->Release();
                    pIResult = NULL;
                                                            
                }

                pIDispResult->Release();
                pIDispResult = NULL;
            }
        }
    }
    catch(COleException *pE)
	{
		ODx("Caught OleException:", pE->m_sc);
		char szErrorMsg[255];

		pE->GetErrorMessage(szErrorMsg, 255, NULL);
		if(szErrorMsg[0])
		{
			ODsz("Error message:", szErrorMsg);
		}
		pE->Delete();
	}
	catch(COleDispatchException *pE)
	{
		ODx("Caught COleDispatchException:", pE->m_wCode);

		if(pE->m_strDescription!="")
		{
			ODsz("Error message:", (LPCTSTR)pE->m_strDescription);
		}
		if(pE->m_strSource!="")
		{
			ODsz("Thrown by:", (LPCTSTR)pE->m_strSource);
		}
		pE->Delete();
	}
	catch(CException *pE)
	{
		ODS("Caught CException");
		char szErrorMsg[255];

		pE->GetErrorMessage(szErrorMsg, 255, NULL);
		if(szErrorMsg[0])
		{
			ODsz("Error message:", szErrorMsg);
		}
		pE->Delete();
	}
	catch(...)
	{
		ODS("caught exception")
	}


    return hRetVal;
}


//----------------------------------------------
// Performing an LDAP Synchronous Search.
//
// Be aware that you must set the command prompt screen buffer 
// height to 350 and the width to 90.
//-------------------------------------------------------------


//-----------------------------------------------------------
// This subroutine must have validated credentials (name and
// password) passed to it.
//-----------------------------------------------------------



//MyLDAPSearch("ws-bruessler02, 712, "dc=meta", 
int MyLDAPSearch(CString sHostName, int iPort, CString sDC, CString sFilter, CString sAttributeName, CStringArray &AttributeNumbers, CStringArray &AttributeNumbersDescriptions, CString sAttributeDescription, ULONG numReturns, CString sSeachString, CString &sName, INameNumberSearchResultCollection* pCollection)
{
    ODsz("Host:        ", (LPCTSTR)sHostName);
    ODsz("Filter:      ", (LPCTSTR)sFilter);
    ODsz("Search for:  ", (LPCTSTR)sSeachString);
    ODi( "Port:        ", iPort);
    ODi( "NumReturns:  ", numReturns);

    bool bFoundSomething=false;

    sFilter.Replace("%searchstring%", sSeachString);
    ODsz("Searchstring:", (LPCTSTR)sFilter);
    
    //---------------------------------------------------------
    // Initialize a session. LDAP_PORT is the default port, 389.
    //---------------------------------------------------------
    LDAP* pLdapConnection = NULL;
    
    pLdapConnection = ldap_init((PCHAR)(LPCTSTR)sHostName, iPort);
    
    if (pLdapConnection == NULL)
    {
        ODx("ldap_init failed with ",LdapGetLastError());
        ldap_unbind(pLdapConnection);
        return -1;
    }
    else
    {
        ODS("ldap_init succeeded");
    }
    
    
    //-------------------------------------------------------
    // Set session options.
    //-------------------------------------------------------
    ULONG version = LDAP_VERSION3;
    ULONG lRtn = 0;
    
    // Set the version to 3.0 (default is 2.0).
    lRtn = ldap_set_option(
                    pLdapConnection,           // Session handle
                    LDAP_OPT_PROTOCOL_VERSION, // Option
                    (void*) &version);         // Option value

    if(lRtn == LDAP_SUCCESS)
    {
        ODS("ldap version set to 3.0");
    }
    else
    {
        ODx("SetOption Error: ", lRtn);
        ldap_unbind(pLdapConnection);
        return -1;
    }

    // Set the limit on the number of entries returned to 10.
    lRtn = ldap_set_option(
                    pLdapConnection,       // Session handle
                    LDAP_OPT_SIZELIMIT,    // Option
                    (void*) &numReturns);  // Option value

    if(lRtn == LDAP_SUCCESS)
    {
        ODi("Max return entries set to ", numReturns);
    }
    else
    {
        ODx("SetOption Error: ", lRtn);
        ldap_unbind(pLdapConnection);
        return -1;
    }
    
    
    //--------------------------------------------------------
    // Connect to the server.
    //--------------------------------------------------------
    lRtn = ldap_connect(pLdapConnection, NULL);
    
    if(lRtn == LDAP_SUCCESS)
    {
        ODS("ldap_connect succeeded");
    }
    else
    {
        ODx("ldap_connect failed with ",lRtn);
        ldap_unbind(pLdapConnection);
        return -1;
    }
    
    
    //--------------------------------------------------------
    // Bind with credentials.
    //--------------------------------------------------------

    /*
    SEC_WINNT_AUTH_IDENTITY secIdent;
 
    secIdent.User = (unsigned char*)pUserName;
    secIdent.UserLength = strlen(pUserName);
    secIdent.Password = (unsigned char*)pPassword;
    secIdent.PasswordLength = strlen(pPassword);
    secIdent.Domain = (unsigned char*)hostName;
    secIdent.DomainLength = strlen(hostName);
    secIdent.Flags = SEC_WINNT_AUTH_IDENTITY_ANSI;

    lRtn = ldap_bind_s(
                pLdapConnection,      // Session Handle
                (PCHAR)(LPCTSTR)sDC,  // Domain DN
                (PCHAR)&secIdent,     // Credential structure
                LDAP_AUTH_SIMPLE);    // Auth mode
    */
    ODsz("ldap_bind_s to ", (LPCTSTR)sDC);
    lRtn = ldap_bind_s(
                pLdapConnection,    // Session Handle
                (PCHAR)(LPCTSTR)sDC,// Domain DN
                NULL,               // Credential structure
                LDAP_AUTH_SIMPLE);  // Auth mode

    if(lRtn == LDAP_SUCCESS)
    {
        ODS("ldap_bind_s succeeded");
    }
    else
    {
        ODx("ldap_bind_s failed with ",lRtn);
        ldap_unbind(pLdapConnection);
        return -1;
    }
	 
    //----------------------------------------------------------
    // Perform a synchronous search of fabrikam.com for 
    // all user objects that have a "person" category.
    //----------------------------------------------------------
    ULONG errorCode = LDAP_SUCCESS;
    LDAPMessage* pSearchResult;

    /*
    PCHAR pMyAttributes[3];

    pMyAttributes[0] = "cn";
    pMyAttributes[1] = "telephonenumber";
    pMyAttributes[2] = NULL;
    
    errorCode = ldap_search_s(
                    pLdapConnection,    // Session handle
                    (PCHAR)(LPCTSTR)sDC,// DN to start search
                    LDAP_SCOPE_SUBTREE, // Scope
                    sFilter,            // Filter
                    pMyAttributes,      // Retrieve list of attributes
                    0,                  // Get both attributes and values
                    &pSearchResult);    // [out] Search results
    */
    
    errorCode = ldap_search_s(
                    pLdapConnection,        // Session handle
                    (PCHAR)(LPCTSTR)sDC,    // DN to start search
                    LDAP_SCOPE_SUBTREE,     // Scope
                    (PCHAR)(LPCTSTR)sFilter,// Filter
                    NULL,                   // Retrieve list of attributes
                    0,                      // Get both attributes and values
                    &pSearchResult);        // [out] Search results
    
    if (errorCode != LDAP_SUCCESS)
    {
        ODx("ldap_search_s failed with ",errorCode);
        ldap_unbind_s(pLdapConnection);
        if(pSearchResult != NULL)
            ldap_msgfree(pSearchResult);
        return -1;
    }
    else
    {
        ODS("ldap_search succeeded \n");
    }
    
    //----------------------------------------------------------
    // Get the number of entries returned.
    //----------------------------------------------------------
    ULONG numberOfEntries;
    
    numberOfEntries = ldap_count_entries(
                        pLdapConnection,    // Session handle
                        pSearchResult);     // Search result
    
    if(numberOfEntries == NULL)
    {
        ODx("ldap_count_entries failed with ",errorCode);
        ldap_unbind_s(pLdapConnection);
        if(pSearchResult != NULL)
            ldap_msgfree(pSearchResult);
        return -1;
    }
    else
    {
        ODS("ldap_count_entries succeeded");
    }
    
    ODi("The number of entries is: ", numberOfEntries);
    
    
    //----------------------------------------------------------
    // Loop through the search entries, get, and output the
    // requested list of attributes and values.
    //----------------------------------------------------------
    LDAPMessage* pEntry = NULL;
    PCHAR pEntryDN = NULL;
    ULONG iCnt = 0;
    char* sMsg;
    BerElement* pBer = NULL;
    PCHAR pAttribute = NULL;
    PCHAR* ppValue = NULL;
    ULONG iValue = 0;
    
    for( iCnt=0; iCnt < numberOfEntries; iCnt++ )
    {
        bool bFoundName=false;
        CStringArray Numbers;
        CStringArray Descriptions;
        CString Description;

        // Get the first/next entry.
        if( !iCnt )
            pEntry = ldap_first_entry(pLdapConnection, pSearchResult);
        else
            pEntry = ldap_next_entry(pLdapConnection, pEntry);
        
        // Output a status message.
        sMsg = (!iCnt ? "ldap_first_entry" : "ldap_next_entry");
        if( pEntry == NULL )
        {
            ODsz("Request ", sMsg);
            ODx("Failed with ", LdapGetLastError());
            ldap_unbind_s(pLdapConnection);
            ldap_msgfree(pSearchResult);
            return -1;
        }
        
        // Output the entry number.
        ODi("ENTRY NUMBER ", iCnt);
                
        // Get the firt attribute name.
        pAttribute = ldap_first_attribute(
                      pLdapConnection,   // Session handle
                      pEntry,            // Current entry
                      &pBer);            // [out] Current BerElement

        // Output the attribute names for the current object
        // and output values.
        while(pAttribute != NULL)
        {
            // Output the attribute name.
            ODsz("     ATTR: ",pAttribute);
            
            // Get the string values.

            ppValue = ldap_get_values(
                          pLdapConnection,  // Session Handle
                          pEntry,           // Current entry
                          pAttribute);      // Current attribute

            // Print status if no values are returned (NULL ptr)
            if(ppValue == NULL)
            {
                ODS(": [NO ATTRIBUTE VALUE RETURNED]");
            }

            // Output the attribute values
            else
            {
                iValue = ldap_count_values(ppValue);
                if(!iValue)
                {
                    ODS(": [BAD VALUE LIST]");
                }
                else
                {
                    // Output the first attribute value
                    ODsz(": ", *ppValue);

                    if(_stricmp(pAttribute, sAttributeName)==0)
                    {
                        sName=*ppValue;
                        bFoundName=true;
                        ODS("Found name");
                        if(!pCollection)
                        {
                            bFoundSomething=true;
                        }
                    }
                    else if((sAttributeDescription.GetLength()>0) && (_stricmp(pAttribute, sAttributeDescription)==0))
                    {
                        Description=*ppValue;
                        ODS("Found description");
                    }
                    else
                    {
                        for(int k=0; k<AttributeNumbers.GetCount(); k++)
                        {
                            if(_stricmp(pAttribute, AttributeNumbers[k])==0)
                            {
                                ODS("Found number");
                                Numbers.Add(*ppValue);
                                Descriptions.Add(AttributeNumbersDescriptions[k]);
                                break;
                            }
                        }
                    }


                    // Output more values if available
                    /*
                    ULONG z;
                    for(z=1; z<iValue; z++)
                    {
                        ODsz(", ", ppValue[z]);
                    }
                    */
                }
            } 

            // Free memory.
            if(ppValue != NULL)  
                ldap_value_free(ppValue);
            ppValue = NULL;
            ldap_memfree(pAttribute);

            if(bFoundName && (!pCollection))
            {
                break;
            }
            
            // Get next attribute name.
            pAttribute = ldap_next_attribute(
                            pLdapConnection,   // Session Handle
                            pEntry,            // Current entry
                            pBer);             // Current BerElement
            ODS("\n");
        }


        if(bFoundName && (Numbers.GetCount()>0) && (pCollection) )
        {
            for(int i=0; i<Numbers.GetCount(); i++)
            {
                ODS("Add search result to list");
                CString desc=Description;
                if(desc.GetLength()>0)
                {
                    desc+=", ";
                }
                desc+=Descriptions[i];
                AddSearchResultToCollection(pCollection, sName, Numbers[i], desc);
            }
        }

        
        if( pBer != NULL )
            ber_free(pBer,0);
        pBer = NULL;

        if(bFoundName && (!pCollection) )
        {
            break;
        }
    }
    
    //----------------------------------------------------------
    // Normal cleanup and exit.
    //----------------------------------------------------------
    ldap_unbind(pLdapConnection);
    ldap_msgfree(pSearchResult);
    ldap_value_free(ppValue);

    if(bFoundSomething)
    {
        return 0;
    }
    else
    {
        return -1;
    }   
}




/////////////////////////////////////////////////////////////////////////////
// CLdapResolver
CLdapResolver::CLdapResolver()
{
    m_dwPort=712;
    m_sServer="";
    m_sFilterSearchInNumber="";
    m_sFilterSearchInName="";
    m_sDC="";
    m_dwNumReturns=10;
    m_sAttributeName="";
    m_sAttributeDescription="";
    m_sAttributeNumbers="";

}



CLdapResolver::~CLdapResolver()
{
}



//IClientResolverAddIn

/*
 * This method will resolve a phone number to a name
 *
 * Input:  Number
 * Output: Name.
 * If the number can be resolved, *pName is the resolved name and the HRESULT is S_OK.
 * If the number cannot be resolved, *pName is set to NULL and the HRESULT is S_FALSE.
 * If Number is NULL or pName is NULL, HRESULT will be E_INVALIDARG
 */
STDMETHODIMP CLdapResolver::ResolveNumber(/*[in]*/ BSTR Number, /*[out]*/ BSTR *pName)
{
    HRESULT         hRetVal=S_FALSE;
    CString         sNumber=(char*)_bstr_t(Number);
    CString         sResolvedName;

    if( (Number!=NULL) && (pName!=NULL) )
    {
        ODsz("ResolveNumber: ", Number);
        if(MyLDAPSearch(m_sServer, m_dwPort, m_sDC, m_sFilterSearchInNumber, m_sAttributeName, m_AttributeNumbers, m_AttributeNumbersDescriptions, m_sAttributeDescription, m_dwNumReturns, sNumber, sResolvedName, NULL)==0)
        {
            *pName=sResolvedName.AllocSysString();
            hRetVal=S_OK;
        }
        else
        {
            *pName=NULL;
            hRetVal=S_FALSE;
        }

    }
    else
    {
        hRetVal=E_INVALIDARG;
    }

    return hRetVal;
}



STDMETHODIMP CLdapResolver::FulltextSearchInContacts(BSTR bstrSearchText, BOOL bEnableSearchInNumbers,  IDispatch* pISearchResultCollection)
{
    HRESULT hRetVal = E_FAIL;
    CString sSearchText=(char*)_bstr_t(bstrSearchText);
    CString sDummy;

    if(pISearchResultCollection)
    {
        if(bstrSearchText)
        {
            ODsz("SearchName: ", bstrSearchText);
        }
        INameNumberSearchResultCollection* pISRC = NULL;

        hRetVal = pISearchResultCollection->QueryInterface(IID_INameNumberSearchResultCollection, (void**) &pISRC);

        if(SUCCEEDED(hRetVal) && pISRC)
        {       
            if(MyLDAPSearch(m_sServer, m_dwPort, m_sDC, m_sFilterSearchInName, m_sAttributeName, m_AttributeNumbers, m_AttributeNumbersDescriptions, m_sAttributeDescription, m_dwNumReturns, sSearchText, sDummy, pISRC)==0)
            {
                hRetVal=S_OK;
            }
            else
            {
                hRetVal=S_FALSE;
            }

            pISRC->Release();
            pISRC = NULL;

            hRetVal = S_OK;
        }
    }
    else
    {
        hRetVal = E_POINTER;
    }

    return hRetVal;
}





/*
 * This method will return the preferred phone number style to be resolved
 *
 *     typedef enum
 *     {
 *         PubCLMgrNumberStyleFull,        //e.g. 0004923147770 or 0023147770
 *         PubCLMgrNumberStylePlain,       //e.g. 4923147770 or 23147770
 *         PubCLMgrNumberStyleCanonical    //e.g. +49 (231) 47770 or +49 (231) 4777-0
 *     } PubCLMgrNumberStyle;
 *
 */
STDMETHODIMP CLdapResolver::GetPreferredNumberStyle(/*[out]*/ DWORD *pStyle)
{
    HRESULT hr=S_OK;

    if(pStyle)
    {
        *pStyle=PubCLMgrNumberStylePlain;
    }
    else
    {
        hr=E_INVALIDARG;
    }

    return hr;
}



//IClientAddInLoader

/*
 * This method will be called after loading the PlugIn
 *
 * Input:  dwReserved, must be NULL
 * If HRESULT is not S_OK, the PlugIn will be released, UnInitialize will not be called in that case
 */
STDMETHODIMP CLdapResolver::Initialize(/*[in]*/ DWORD dwReserved)
{
    HRESULT hRetVal=S_OK;
    HRESULT hr;
    IClientLineMgrPub *pIClientLineMgrPub=NULL;


    CString sPlugInSection;
    BSTR    bstrCLSID;
    CString sGuid;
    CString sCompany;

    m_sServer="";
    m_dwPort=712;
    m_dwNumReturns=10;
    m_sDC="dc=meta";

    m_sFilterSearchInNumber=CString("(|(telephonenumber=*%searchstring%)(mobile=*%searchstring%)(otherTelephone=*%searchstring%)(homephone=*%searchstring%))");
    m_sFilterSearchInName=CString("(|(displayName=*%searchstring%*)(givenName=*%searchstring%*)(sn=*%searchstring%*))");

    m_sAttributeName="displayName";
    m_sAttributeNumbers="telephonenumber=Phone;mobile=Mobile;otherTelephone=Other Phone;homephone=Home";
    m_sAttributeDescription="company";


    if(StringFromCLSID(CLSID_LdapResolver, &bstrCLSID)==S_OK)
    {
        sGuid=(char*)_bstr_t(bstrCLSID);
        sCompany.LoadStringA(IDS_COMPANY);
        sPlugInSection=CString("SOFTWARE\\")+sCompany+CString("\\Client Line Manager\\CurrentVersion\\Options\\PlugIns\\")+sGuid;

        HKEY hKey=NULL;

        if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, sPlugInSection, 0, KEY_QUERY_VALUE, &hKey)==ERROR_SUCCESS)
        {
            ODS("Opened registry");

            DWORD dwPort=0;
            DWORD dwType=0;
            DWORD dwSize=0;

            dwSize=sizeof(DWORD);

            if(RegQueryValueEx(hKey, "Port", 0, &dwType, (LPBYTE)&dwPort, &dwSize)==ERROR_SUCCESS)
            {
                ODS("Read: Port");
                if(dwType==REG_DWORD)
                {
                    m_dwPort=dwPort;
                }
            }


            if(RegQueryValueEx(hKey, "NumReturns", 0, &dwType, (LPBYTE)&dwPort, &dwSize)==ERROR_SUCCESS)
            {
                ODS("Read: NumReturns");
                if(dwType==REG_DWORD)
                {
                    m_dwNumReturns=dwPort;
                }
            }


            char szBuffer[30000];

            dwSize=sizeof(szBuffer);
            if(RegQueryValueEx(hKey, "Server", 0, &dwType, (LPBYTE)szBuffer, &dwSize)==ERROR_SUCCESS)
            {
                ODS("Read: Server");
                if(dwType==REG_SZ)
                {
                    m_sServer=szBuffer;
                }
            }


            dwSize=sizeof(szBuffer);
            if(RegQueryValueEx(hKey, "DC", 0, &dwType, (LPBYTE)szBuffer, &dwSize)==ERROR_SUCCESS)
            {
                ODS("Read: DC");
                if(dwType==REG_SZ)
                {
                    m_sDC=szBuffer;
                }
            }


            dwSize=sizeof(szBuffer);
            if(RegQueryValueEx(hKey, "FilterSearchInNumber", 0, &dwType, (LPBYTE)szBuffer, &dwSize)==ERROR_SUCCESS)
            {
                ODS("Read: FilterSearchInNumber");
                if(dwType==REG_SZ)
                {
                    m_sFilterSearchInNumber=szBuffer;
                }
            }


            dwSize=sizeof(szBuffer);
            if(RegQueryValueEx(hKey, "FilterSearchInName", 0, &dwType, (LPBYTE)szBuffer, &dwSize)==ERROR_SUCCESS)
            {
                ODS("Read: FilterSearchInName");
                if(dwType==REG_SZ)
                {
                    m_sFilterSearchInName=szBuffer;
                }
            }


            dwSize=sizeof(szBuffer);
            if(RegQueryValueEx(hKey, "NameField", 0, &dwType, (LPBYTE)szBuffer, &dwSize)==ERROR_SUCCESS)
            {
                ODS("Read: NameField");
                if(dwType==REG_SZ)
                {
                    m_sAttributeName=szBuffer;
                }
            }


            dwSize=sizeof(szBuffer);
            if(RegQueryValueEx(hKey, "DescriptionField", 0, &dwType, (LPBYTE)szBuffer, &dwSize)==ERROR_SUCCESS)
            {
                ODS("Read: DescriptionField");
                if(dwType==REG_SZ)
                {
                    m_sAttributeDescription=szBuffer;
                }
            }


            dwSize=sizeof(szBuffer);
            if(RegQueryValueEx(hKey, "NumberFields", 0, &dwType, (LPBYTE)szBuffer, &dwSize)==ERROR_SUCCESS)
            {
                ODS("Read: NumberFields");
                if(dwType==REG_SZ)
                {
                    m_sAttributeNumbers=szBuffer;
                }
            }

            RegCloseKey(hKey);
        }
    }

    m_AttributeNumbers.RemoveAll();
    m_AttributeNumbersDescriptions.RemoveAll();


    CString entry;
    CString name;
    CString description;

    int iCurPosEntries=0;

    entry=m_sAttributeNumbers.Tokenize(_T(";"), iCurPosEntries);
    while(entry!=_T(""))
    {
        ODsz("Number field entry: ", (LPCTSTR)entry);
    
        int iCurPosEntry=0;

        name="";
        description="";

        name=entry.Tokenize(_T("="), iCurPosEntry);
        if(name!=_T(""))
        {
            description=entry.Tokenize(_T("="), iCurPosEntry);

            if(description!=_T(""))
            {
                ODsz("Field name: ", (LPCTSTR)name);
                ODsz("Field description: ", (LPCTSTR)description);
                m_AttributeNumbers.Add(name);
                m_AttributeNumbersDescriptions.Add(description);
            }
        }

        entry=m_sAttributeNumbers.Tokenize(_T(";"), iCurPosEntries);
    }


    return hRetVal;
}



/*
 * This method will return the visible name of the PlugIn
 *
 */
STDMETHODIMP CLdapResolver::GetName(/*[out]*/ BSTR *pName)
{
    HRESULT hr=S_OK;

    if(pName)
    {
        CString sName;

        sName.LoadString(IDR_MAINFRAME);

        if(sName.IsEmpty())
        {
            sName="Unknown";
        }

        *pName=sName.AllocSysString();
    }
    else
    {
        hr=E_INVALIDARG;
    }

    return hr;
}



/*
 * This method will return the version of the PlugIn
 * Format: e.g. "3.2.0.4"   for Version 3.20 build 4
 */
STDMETHODIMP CLdapResolver::GetVersion(/*[out]*/ BSTR *pVersion)
{
    HRESULT hr=S_OK;

    if(pVersion)
    {
        CString sName="Unknown";

        *pVersion=sName.AllocSysString();
    }
    else
    {
        hr=E_INVALIDARG;
    }

    return hr;
}



/*
 * This method will be called before releasing the PlugIn
 *
 * Input:  dwReserved, must be NULL
 */
STDMETHODIMP CLdapResolver::UnInitialize(/*[in]*/ DWORD dwReserved)
{
    return S_OK;
}



