/*---------------------------------------------------------------------------

   Swyx Solutions
   Joseph-von-Fraunhofer-Str. 13a
   44227 Dortmund

  ---------------------------------------------------------------------------

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (C) 2007 Swyx Solutions

  All Rights Reserved.

  ---------------------------------------------------------------------------

     Project:   Swyx Client SDK

      Module:   Visual C++ ATL Client PlugIn sample
 
    Comments:   

----------------------------------------------------------------------------*/




// LdapResolver.h : Declaration of the CLdapResolver

#ifndef __LdapResolver_H_
#define __LdapResolver_H_

#include "resource.h"       // main symbols



extern _ATL_FUNC_INFO DispOnLineMgrNotificationInfo;


/////////////////////////////////////////////////////////////////////////////
// CLdapResolver
class ATL_NO_VTABLE CLdapResolver : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CLdapResolver, &CLSID_LdapResolver>,
    public IClientAddInLoader,
    public IClientResolverAddIn,
    public IFulltextSearchAddIn
{
public:
	CLdapResolver();
	virtual ~CLdapResolver();
    HRESULT SendSearchResult(INameNumberSearchResultCollection* pISRC, CString szName, CString szNumber, CString szDescription);


    DWORD           m_dwPort;
    DWORD           m_dwNumReturns;
    CString         m_sServer;
    CString         m_sDC;
    CString         m_sFilterSearchInNumber;
    CString         m_sFilterSearchInName;
    CString         m_sAttributeName;
    CString         m_sAttributeDescription;
    CString         m_sAttributeNumbers;
    CStringArray    m_AttributeNumbers;
    CStringArray    m_AttributeNumbersDescriptions;


DECLARE_REGISTRY_RESOURCEID(IDR_LdapResolver)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CLdapResolver)
	COM_INTERFACE_ENTRY(IClientAddInLoader)
	COM_INTERFACE_ENTRY(IClientResolverAddIn)   
    COM_INTERFACE_ENTRY(IFulltextSearchAddIn)   
END_COM_MAP()




//IClientResolverAddIn
public:
    /*
     * This method will resolve a phone number to a name
     *
     * Input:  Number
     * Output: Name.
     * If the number can be resolved, *pName is the resolved name and the HRESULT is S_OK.
     * If the number cannot be resolved, *pName is set to NULL and the HRESULT is S_FALSE.
     * If Number is NULL or pName is NULL, HRESULT will be E_INVALIDARG
     */
	STDMETHOD(ResolveNumber)(/*[in]*/ BSTR Number, /*[out]*/ BSTR *pName);

    /*
     * This method will return the preferred phone number style to be resolved
     *
     *     typedef enum
     *     {
     *         PubCLMgrNumberStyleFull,        //e.g. 0004923147770 or 0023147770
     *         PubCLMgrNumberStylePlain,       //e.g. 4923147770 or 23147770
     *         PubCLMgrNumberStyleCanonical    //e.g. +49 (231) 47770 or +49 (231) 4777-0
     *     } PubCLMgrNumberStyle;
     *
     */
	STDMETHOD(GetPreferredNumberStyle)(/*[out]*/ DWORD *pStyle);


//IClientAddInLoader
public:
    /*
     * This method will be called after loading the PlugIn
     *
     * Input:  dwReserved, must be NULL
     * If HRESULT is not S_OK, the PlugIn will be released, UnInitialize will not be called in that case
     */
	STDMETHOD(Initialize)(/*[in]*/ DWORD dwReserved);

    /*
     * This method will return the visible name of the PlugIn
     *
     */
	STDMETHOD(GetName)(/*[out]*/ BSTR *pName);

    /*
     * This method will return the version of the PlugIn
     * Format: e.g. "3.2.0.4"   for Version 3.20 build 4
     */
	STDMETHOD(GetVersion)(/*[out]*/ BSTR *pVersion);

    /*
     * This method will be called before releasing the PlugIn
     *
     * Input:  dwReserved, must be NULL
     */
	STDMETHOD(UnInitialize)(/*[in]*/ DWORD dwReserved);


    STDMETHOD(FulltextSearchInContacts)(/*[in]*/ BSTR bstrSearchText, /*[in]*/ BOOL bEnableSearchInNumbers, /*[in]*/ IDispatch* pISearchResultCollection);        

};

#endif //__LdapResolver_H_
